@@ -24,8 +24,12 @@
 #include "debug.h"
 #include "fileinfo.h"
 #include "dir.h"
+#include "growbuf.h"
+#include "entry.h"
+#include "commentscan.h"
 
 #include <map>
+#include <unordered_map>
 #include <string>
 #include <fstream>
 
@@ -45,8 +49,8 @@ class CiteInfoImpl : public CiteInfo
     CiteInfoImpl(const QCString &label, const QCString &text=QCString())
     : m_label(label), m_text(text) { }
 
-    virtual QCString label()    const { return m_label;    }
-    virtual QCString text()     const { return m_text;     }
+    QCString label() const override { return m_label;    }
+    QCString text()  const override { return m_text;     }
 
     void setText(const QCString &s) { m_text = s; }
 
@@ -58,6 +62,7 @@ class CiteInfoImpl : public CiteInfo
 struct CitationManager::Private
 {
   std::map< std::string,std::unique_ptr<CiteInfoImpl> > entries;
+  std::unordered_map< int,std::string > formulaCite;
 };
 
 CitationManager &CitationManager::instance()
@@ -86,7 +91,7 @@ const CiteInfo *CitationManager::find(const QCString &label) const
   {
     return it->second.get();
   }
-  return 0;
+  return nullptr;
 }
 
 void CitationManager::clear()
@@ -136,7 +141,7 @@ void CitationManager::insertCrossReferencesForBibFile(const QCString &bibFile)
   std::string lineStr;
   while (getline(f,lineStr))
   {
-    int i;
+    int i = -1;
     QCString line(lineStr);
     if (line.stripWhiteSpace().startsWith("@"))
     {
@@ -183,7 +188,7 @@ void CitationManager::insertCrossReferencesForBibFile(const QCString &bibFile)
       int k = line.find('}',i);
       if (j>i && k>j)
       {
-        QCString crossrefName = line.mid(static_cast<size_t>(j+1),static_cast<uint>(k-j-1));
+        QCString crossrefName = line.mid(static_cast<size_t>(j+1),static_cast<uint32_t>(k-j-1));
         // check if the reference with the cross reference is used
         // insert cross reference when cross reference has not yet been added.
         if ((p->entries.find(citeName.str())!=p->entries.end()) &&
@@ -196,6 +201,103 @@ void CitationManager::insertCrossReferencesForBibFile(const QCString &bibFile)
   }
 }
 
+const std::string g_formulaMarker = "CITE_FORMULA_";
+
+QCString CitationManager::getFormulas(const QCString &s)
+{
+  if (s.isEmpty()) return s;
+  GrowBuf growBuf;
+  GrowBuf formulaBuf;
+  bool insideFormula = false;
+  int citeFormulaCnt = 1;
+  const size_t tmpLen = 30;
+  char tmp[tmpLen];
+  const char *ps=s.data();
+  char c = 0;
+  while ((c=*ps++))
+  {
+    if (insideFormula)
+    {
+      switch (c)
+      {
+        case '\\':
+          formulaBuf.addChar(c);
+          c = *ps++;
+          formulaBuf.addChar(c);
+          break;
+        case '\n':
+          formulaBuf.addChar(c);
+          formulaBuf.addChar(0);
+          growBuf.addChar('$');
+          growBuf.addStr(formulaBuf.get());
+          insideFormula = false;
+          formulaBuf.clear();
+          break;
+        case '$':
+          qsnprintf(tmp,tmpLen,"%s%06d",g_formulaMarker.c_str(),citeFormulaCnt);
+          formulaBuf.addChar(0);
+          p->formulaCite.emplace(citeFormulaCnt,std::string("\\f$") + formulaBuf.get() + "\\f$");
+          citeFormulaCnt++;
+          // need { and } due to the capitalization rules of bibtex.
+          growBuf.addChar('{');
+          growBuf.addStr(tmp);
+          growBuf.addChar('}');
+          insideFormula = false;
+          formulaBuf.clear();
+          break;
+        default:
+          formulaBuf.addChar(c);
+          break;
+      }
+    }
+    else
+    {
+      switch (c)
+      {
+        case '\\':
+          growBuf.addChar(c);
+          c = *ps++;
+          growBuf.addChar(c);
+          break;
+        case '$':
+          insideFormula = true;
+          break;
+        default:
+          growBuf.addChar(c);
+          break;
+      }
+    }
+  }
+  if (insideFormula)
+  {
+    formulaBuf.addChar(0);
+    growBuf.addStr(formulaBuf.get());
+    formulaBuf.clear();
+  }
+  growBuf.addChar(0);
+  return growBuf.get();
+}
+
+QCString CitationManager::replaceFormulas(const QCString &s)
+{
+  if (s.isEmpty()) return s;
+  QCString t;
+  int pos=0;
+  int i = -1;
+  while ((i=s.find(g_formulaMarker.c_str(),pos))!=-1)
+  {
+    t += s.mid(pos,i-pos);
+    int markerSize = static_cast<int>( g_formulaMarker.length());
+    int markerId = atoi(s.mid(i+markerSize,6).data());
+    auto it = p->formulaCite.find(markerId);
+    if (it != p->formulaCite.end()) t += it->second;
+    pos = i + markerSize+6;
+  }
+  t += s.mid(pos);
+  //printf("replaceFormulas(%s)=%s\n",qPrint(s),qPrint(t));
+  return t;
+}
+
 void CitationManager::generatePage()
 {
   //printf("** CitationManager::generatePage() count=%d\n",m_ordering.count());
@@ -265,7 +367,26 @@ void CitationManager::generatePage()
       if (!bibFile.isEmpty())
       {
         ++i;
-        copyFile(bibFile,bibOutputDir + bibTmpFile + QCString().setNum(i) + ".bib");
+        std::ifstream f_org = Portable::openInputStream(bibFile);
+        if (!f_org.is_open())
+        {
+          err("could not open file %s for reading\n",qPrint(bibFile));
+        }
+        std::ofstream f_out = Portable::openOutputStream(bibOutputDir + bibTmpFile + QCString().setNum(i) + ".bib");
+        if (!f_out.is_open())
+        {
+          err("could not open file %s for reading\n",qPrint(bibOutputDir + bibTmpFile + QCString().setNum(i) + ".bib"));
+        }
+        QCString docs;
+        std::string lineStr;
+        while (getline(f_org,lineStr))
+        {
+          docs += lineStr + "\n";
+        }
+        docs = getFormulas(docs);
+        f_out << docs;
+        if (f_org.is_open()) f_org.close();
+        if (f_out.is_open()) f_out.close();
         bibOutputFiles = bibOutputFiles + " " + bibTmpDir + bibTmpFile + QCString().setNum(i) + ".bib";
       }
     }
@@ -276,10 +397,10 @@ void CitationManager::generatePage()
 
   // 5. run bib2xhtml perl script on the generated file which will insert the
   //    bibliography in citelist.doc
-  int exitCode;
   QCString perlArgs = "\""+bib2xhtmlFile+"\" "+bibOutputFiles+" \""+ citeListFile+"\"";
   if (citeDebug) perlArgs+=" -d";
-  if ((exitCode=Portable::system("perl",perlArgs)) != 0)
+  int exitCode = Portable::system("perl",perlArgs);
+  if (exitCode!=0)
   {
     err("Problems running bibtex. Verify that the command 'perl --version' works from the command line. Exit code: %d\n",
         exitCode);
@@ -332,10 +453,38 @@ void CitationManager::generatePage()
     //printf("doc=[%s]\n",qPrint(doc));
   }
 
-  // 7. add it as a page
+  // 7. place formulas back and run the conversion of \f$ ... \f$ to the internal required format
+  {
+    doc = replaceFormulas(doc);
+    Entry            current;
+    bool             needsEntry = false;
+    CommentScanner   commentScanner;
+    int              lineNr = 0;
+    int              pos = 0;
+    GuardedSectionStack guards;
+    Protection       prot = Protection::Public;
+    commentScanner.parseCommentBlock(
+        nullptr,
+        &current,
+        doc,          // text
+        fileName(),   // file
+        lineNr,       // line of block start
+        false,        // isBrief
+        false,        // isJavaDocStyle
+        false,        // isInBody
+        prot,         // protection
+        pos,          // position,
+        needsEntry,
+        false,
+        &guards
+        );
+    doc = current.doc;
+  }
+
+  // 8. add it as a page
   addRelatedPage(fileName(),theTranslator->trCiteReferences(),doc,fileName(),1,1);
 
-  // 8. for latex we just copy the bib files to the output and let
+  // 9. for latex we just copy the bib files to the output and let
   //    latex do this work.
   if (Config_getBool(GENERATE_LATEX))
   {
@@ -363,7 +512,7 @@ void CitationManager::generatePage()
     }
   }
 
-  // 9. Remove temporary files
+  // 10. Remove temporary files
   if (!citeDebug)
   {
     thisDir.remove(citeListFile.str());
@@ -374,7 +523,7 @@ void CitationManager::generatePage()
     // so no problem.
     for (size_t j = 1; j <= citeDataList.size(); j++)
     {
-      QCString bibFile = bibOutputDir + bibTmpFile + QCString().setNum(static_cast<ulong>(j)) + ".bib";
+      QCString bibFile = bibOutputDir + bibTmpFile + QCString().setNum(static_cast<int>(j)) + ".bib";
       thisDir.remove(bibFile.str());
     }
     thisDir.rmdir(bibOutputDir.str());
