@@ -1,3 +1,17 @@
+/******************************************************************************
+ *
+ * Copyright (C) 1997-2023 by Dimitri van Heesch.
+ *
+ * Permission to use, copy, modify, and distribute this software and its
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
+ * for any purpose. It is provided "as is" without express or implied warranty.
+ * See the GNU General Public License for more details.
+ *
+ * Documents produced by Doxygen are derivative works derived from the
+ * input used in their production; they are not affected by this license.
+ *
+ */
 
 #include <algorithm>
 #include <iterator>
@@ -8,6 +22,7 @@
 #include <stdlib.h>
 #include <assert.h>
 
+#include "anchor.h"
 #include "md5.h"
 #include "regex.h"
 #include "config.h"
@@ -31,7 +46,6 @@
 #include "filedef.h"
 #include "dirdef.h"
 #include "pagedef.h"
-#include "bufstr.h"
 #include "reflist.h"
 #include "utf8.h"
 #include "indexlist.h"
@@ -39,27 +53,58 @@
 
 //-----------------------------------------------------------------------------------------
 
+//! Helper class add copy/assignment support to std::unique_ptr by making a deep copy
+//! Note that T may not be a polymorphic type
+template<class T>
+class DeepCopyUniquePtr : public std::unique_ptr<T>
+{
+  public:
+    using std::unique_ptr<T>::unique_ptr;
+    DeepCopyUniquePtr(const DeepCopyUniquePtr &other)
+       : std::unique_ptr<T>(other ? new T(*other) : nullptr)
+    {
+    }
+    DeepCopyUniquePtr &operator=(const DeepCopyUniquePtr &other)
+    {
+      if (*this!=other) this->reset(other ? new T(*other) : nullptr);
+      return *this;
+    }
+    DeepCopyUniquePtr(DeepCopyUniquePtr &&other) : std::unique_ptr<T>(std::move(other)) {}
+    DeepCopyUniquePtr &operator=(DeepCopyUniquePtr &&other)
+    {
+      std::unique_ptr<T>::operator=(std::move(other));
+      return *this;
+    }
+   ~DeepCopyUniquePtr() = default;
+};
+
+//! Helper to create an object wrapped in a DeepCopyUniquePtr.
+template<typename T, typename... Args>
+DeepCopyUniquePtr<T> make_DeepCopyUnique(Args&&... args)
+{
+  return DeepCopyUniquePtr<T>(new T(std::forward<Args>(args)...));
+}
+
 /** Private data associated with a Symbol DefinitionImpl object. */
 class DefinitionImpl::IMPL
 {
   public:
-   ~IMPL();
     void init(const QCString &df, const QCString &n);
     void setDefFileName(const QCString &df);
 
-    Definition *def = 0;
+    Definition *def = nullptr;
 
     SectionRefs sectionRefs;
 
-    std::unordered_map<std::string,const MemberDef *> sourceRefByDict;
-    std::unordered_map<std::string,const MemberDef *> sourceRefsDict;
+    std::unordered_map<std::string,MemberDef *> sourceRefByDict;
+    std::unordered_map<std::string,MemberDef *> sourceRefsDict;
     RefItemVector xrefListItems;
     GroupList partOfGroups;
 
-    DocInfo   *details = 0;    // not exported
-    DocInfo   *inbodyDocs = 0; // not exported
-    BriefInfo *brief = 0;      // not exported
-    BodyInfo  *body = 0;       // not exported
+    DeepCopyUniquePtr<DocInfo>   details;    // not exported
+    DeepCopyUniquePtr<DocInfo>   inbodyDocs; // not exported
+    DeepCopyUniquePtr<BriefInfo> brief;      // not exported
+    DeepCopyUniquePtr<BodyInfo>  body;       // not exported
     QCString   briefSignatures;
     QCString   docSignatures;
 
@@ -71,14 +116,15 @@ class DefinitionImpl::IMPL
     bool hidden = FALSE;
     bool isArtificial = FALSE;
     bool isAnonymous = FALSE;
+    bool isExported = FALSE;
 
-    Definition *outerScope = 0;  // not owner
+    Definition *outerScope = nullptr;  // not owner
 
     // where the item was defined
     QCString defFileName;
     QCString defFileExt;
 
-    SrcLangExt lang = SrcLangExt_Unknown;
+    SrcLangExt lang = SrcLangExt::Unknown;
 
     QCString id; // clang unique id
 
@@ -88,19 +134,11 @@ class DefinitionImpl::IMPL
     int defLine;
     int defColumn;
 
-    mutable MemberVector referencesMembers;
-    mutable MemberVector referencedByMembers;
+    MemberVector referencesMembers;    // cache for getReferencesMembers()
+    MemberVector referencedByMembers;  // cache for getReferencedByMembers()
 };
 
 
-DefinitionImpl::IMPL::~IMPL()
-{
-  delete brief;
-  delete details;
-  delete body;
-  delete inbodyDocs;
-}
-
 void DefinitionImpl::IMPL::setDefFileName(const QCString &df)
 {
   defFileName = df;
@@ -126,16 +164,17 @@ void DefinitionImpl::IMPL::init(const QCString &df, const QCString &n)
   }
   //printf("localName=%s\n",qPrint(localName));
 
-  brief           = 0;
-  details         = 0;
-  body            = 0;
-  inbodyDocs      = 0;
+  brief.reset();
+  details.reset();
+  body.reset();
+  inbodyDocs.reset();
   sourceRefByDict.clear();
   sourceRefsDict.clear();
   outerScope      = Doxygen::globalScope;
   hidden          = FALSE;
   isArtificial    = FALSE;
-  lang            = SrcLangExt_Unknown;
+  isExported      = FALSE;
+  lang            = SrcLangExt::Unknown;
 }
 
 void DefinitionImpl::setDefFile(const QCString &df,int defLine,int defCol)
@@ -226,8 +265,8 @@ DefinitionImpl::DefinitionImpl(Definition *def,
                        const QCString &df,int dl,int dc,
                        const QCString &name,const char *b,
                        const char *d,bool isSymbol)
+  : m_impl(std::make_unique<DefinitionImpl::IMPL>())
 {
-  m_impl = new DefinitionImpl::IMPL;
   setName(name);
   m_impl->def = def;
   m_impl->defLine = dl;
@@ -244,30 +283,8 @@ DefinitionImpl::DefinitionImpl(Definition *def,
 }
 
 DefinitionImpl::DefinitionImpl(const DefinitionImpl &d)
+  : m_impl(std::make_unique<DefinitionImpl::IMPL>(*d.m_impl))
 {
-  m_impl = new DefinitionImpl::IMPL;
-  *m_impl = *d.m_impl;
-  m_impl->brief = 0;
-  m_impl->details = 0;
-  m_impl->body = 0;
-  m_impl->inbodyDocs = 0;
-  if (d.m_impl->brief)
-  {
-    m_impl->brief = new BriefInfo(*d.m_impl->brief);
-  }
-  if (d.m_impl->details)
-  {
-    m_impl->details = new DocInfo(*d.m_impl->details);
-  }
-  if (d.m_impl->body)
-  {
-    m_impl->body = new BodyInfo(*d.m_impl->body);
-  }
-  if (d.m_impl->inbodyDocs)
-  {
-    m_impl->inbodyDocs = new DocInfo(*d.m_impl->inbodyDocs);
-  }
-
   if (m_impl->isSymbol) addToMap(m_impl->name,m_impl->def);
 }
 
@@ -277,8 +294,6 @@ DefinitionImpl::~DefinitionImpl()
   {
     removeFromMap(m_impl->symbolName,m_impl->def);
   }
-  delete m_impl;
-  m_impl=0;
 }
 
 void DefinitionImpl::setName(const QCString &name)
@@ -316,11 +331,11 @@ void DefinitionImpl::addSectionsToDefinition(const std::vector<const SectionInfo
     SectionManager &sm = SectionManager::instance();
     SectionInfo *gsi=sm.find(si->label());
     //printf("===== label=%s gsi=%p\n",qPrint(si->label),gsi);
-    if (gsi==0)
+    if (gsi==nullptr)
     {
       gsi = sm.add(*si);
     }
-    if (m_impl->sectionRefs.find(gsi->label())==0)
+    if (m_impl->sectionRefs.find(gsi->label())==nullptr)
     {
       m_impl->sectionRefs.add(gsi);
       gsi->setDefinition(m_impl->def);
@@ -335,7 +350,7 @@ bool DefinitionImpl::hasSections() const
   if (m_impl->sectionRefs.empty()) return FALSE;
   for (const SectionInfo *si : m_impl->sectionRefs)
   {
-    if (isSection(si->type()))
+    if (si->type().isSection())
     {
       return TRUE;
     }
@@ -352,37 +367,39 @@ void DefinitionImpl::addSectionsToIndex()
   {
     const SectionInfo *si = *it;
     SectionType type = si->type();
-    if (isSection(type))
+    if (type.isSection())
     {
       //printf("  level=%d title=%s\n",level,qPrint(si->title));
-      int nextLevel = static_cast<int>(type);
-      int i;
+      int nextLevel = type.level();
       if (nextLevel>level)
       {
-        for (i=level;i<nextLevel;i++)
+        for (int i=level;i<nextLevel;i++)
         {
           Doxygen::indexList->incContentsDepth();
         }
       }
       else if (nextLevel<level)
       {
-        for (i=nextLevel;i<level;i++)
+        for (int i=nextLevel;i<level;i++)
         {
           Doxygen::indexList->decContentsDepth();
         }
       }
       QCString title = si->title();
       if (title.isEmpty()) title = si->label();
+      const MemberDef *md = m_impl->def->definitionType()==Definition::TypeMember ? toMemberDef(m_impl->def) : nullptr;
+      const Definition *scope = m_impl->def->definitionType()==Definition::TypeMember ? m_impl->def->getOuterScope() : m_impl->def;
+      title = parseCommentAsText(scope,md,title,si->fileName(),si->lineNr());
       // determine if there is a next level inside this item, but be aware of the anchor and table section references.
       auto it_next = std::next(it);
       bool isDir = (it_next!=m_impl->sectionRefs.end()) ?
-                       (isSection((*it_next)->type()) && static_cast<int>((*it_next)->type()) > nextLevel) : FALSE;
+                       ((*it_next)->type().isSection() && (*it_next)->type().level() > nextLevel) : false;
       Doxygen::indexList->addContentsItem(isDir,title,
                                          getReference(),
                                          m_impl->def->getOutputFileBase(),
                                          si->label(),
-                                         FALSE,
-                                         TRUE);
+                                         false,
+                                         true);
       level = nextLevel;
     }
   }
@@ -400,11 +417,13 @@ void DefinitionImpl::writeDocAnchorsToTagFile(TextStream &tagFile) const
     //printf("%s: writeDocAnchorsToTagFile(%d)\n",qPrint(name()),m_impl->sectionRef.size());
     for (const SectionInfo *si : m_impl->sectionRefs)
     {
-      if (!si->generated() && si->ref().isEmpty() && !si->label().startsWith("autotoc_md"))
+      if (!si->generated() && si->ref().isEmpty() && !AnchorGenerator::instance().isGenerated(si->label().str()))
       {
         //printf("write an entry!\n");
         if (m_impl->def->definitionType()==Definition::TypeMember) tagFile << "  ";
-        tagFile << "    <docanchor file=\"" << addHtmlExtensionIfMissing(si->fileName()) << "\"";
+        QCString fn = si->fileName();
+        addHtmlExtensionIfMissing(fn);
+        tagFile << "    <docanchor file=\"" << fn << "\"";
         if (!si->title().isEmpty())
         {
           tagFile << " title=\"" << convertToXML(si->title()) << "\"";
@@ -417,12 +436,12 @@ void DefinitionImpl::writeDocAnchorsToTagFile(TextStream &tagFile) const
 
 bool DefinitionImpl::_docsAlreadyAdded(const QCString &doc,QCString &sigList)
 {
-  uchar md5_sig[16];
+  uint8_t md5_sig[16];
   char sigStr[33];
   // to avoid mismatches due to differences in indenting, we first remove
   // double whitespaces...
   QCString docStr = doc.simplifyWhiteSpace();
-  MD5Buffer(docStr.data(),docStr.length(),md5_sig);
+  MD5Buffer(docStr.data(),static_cast<unsigned int>(docStr.length()),md5_sig);
   MD5SigToString(md5_sig,sigStr);
   //printf("%s:_docsAlreadyAdded doc='%s' sig='%s' docSigs='%s'\n",
   //    qPrint(name()),qPrint(doc),qPrint(sigStr),qPrint(sigList));
@@ -454,9 +473,9 @@ void DefinitionImpl::_setDocumentation(const QCString &d,const QCString &docFile
   if (!_docsAlreadyAdded(doc,m_impl->docSignatures))
   {
     //printf("setting docs for %s: '%s'\n",qPrint(name()),qPrint(m_doc));
-    if (m_impl->details==0)
+    if (!m_impl->details)
     {
-      m_impl->details = new DocInfo;
+      m_impl->details = make_DeepCopyUnique<DocInfo>();
     }
     if (m_impl->details->doc.isEmpty()) // fresh detailed description
     {
@@ -496,7 +515,7 @@ void DefinitionImpl::_setBriefDescription(const QCString &b,const QCString &brie
   brief = stripLeadingAndTrailingEmptyLines(brief,briefLine);
   brief = brief.stripWhiteSpace();
   if (brief.isEmpty()) return;
-  uint bl = brief.length();
+  size_t bl = brief.length();
   if (bl>0)
   {
     if (!theTranslator || theTranslator->needsPunctuation()) // add punctuation if needed
@@ -522,9 +541,9 @@ void DefinitionImpl::_setBriefDescription(const QCString &b,const QCString &brie
     else
     {
       //fprintf(stderr,"DefinitionImpl::setBriefDescription(%s,%s,%d)\n",b,briefFile,briefLine);
-      if (m_impl->brief==0)
+      if (!m_impl->brief)
       {
-        m_impl->brief = new BriefInfo;
+        m_impl->brief = make_DeepCopyUnique<BriefInfo>();
       }
       m_impl->brief->doc=brief;
       if (briefLine!=-1)
@@ -553,9 +572,9 @@ void DefinitionImpl::setBriefDescription(const QCString &b,const QCString &brief
 
 void DefinitionImpl::_setInbodyDocumentation(const QCString &doc,const QCString &inbodyFile,int inbodyLine)
 {
-  if (m_impl->inbodyDocs==0)
+  if (!m_impl->inbodyDocs)
   {
-    m_impl->inbodyDocs = new DocInfo;
+    m_impl->inbodyDocs = make_DeepCopyUnique<DocInfo>();
   }
   if (m_impl->inbodyDocs->doc.isEmpty()) // fresh inbody docs
   {
@@ -595,7 +614,7 @@ class FilterCache
     //! buffer \a str. Applies filtering if FILTER_SOURCE_FILES is enabled and the file extension
     //! matches a filter. Caches file information so that subsequent extraction of blocks from
     //! the same file can be performed efficiently
-    bool getFileContents(const QCString &fileName,size_t startLine,size_t endLine, BufStr &str)
+    bool getFileContents(const QCString &fileName,size_t startLine,size_t endLine, std::string &str)
     {
       std::lock_guard<std::mutex> lock(m_mutex);
       bool filterSourceFiles = Config_getBool(FILTER_SOURCE_FILES);
@@ -606,7 +625,7 @@ class FilterCache
     }
   private:
     bool getFileContentsPipe(const QCString &fileName,const QCString &filter,
-                             size_t startLine,size_t endLine,BufStr &str)
+                             size_t startLine,size_t endLine,std::string &str)
     {
       auto it = m_cache.find(fileName.str());
       if (it!=m_cache.end()) // cache hit: reuse stored result
@@ -633,21 +652,19 @@ class FilterCache
         QCString cmd=filter+" \""+fileName+"\"";
         Debug::print(Debug::ExtCmd,0,"Executing popen(`%s`)\n",qPrint(cmd));
         FILE *f = Portable::popen(cmd,"r");
-        if (f==0)
+        if (f==nullptr)
         {
           // handle error
           err("Error opening filter pipe command '%s'\n",qPrint(cmd));
-          str.addChar('\0');
           return false;
         }
         FILE *bf = Portable::fopen(Doxygen::filterDBFileName,"a+b");
         FilterCacheItem item;
         item.filePos = m_endPos;
-        if (bf==0)
+        if (bf==nullptr)
         {
           // handle error
           err("Error opening filter database file %s\n",qPrint(Doxygen::filterDBFileName));
-          str.addChar('\0');
           Portable::pclose(f);
           return false;
         }
@@ -664,15 +681,13 @@ class FilterCache
             // handle error
             err("Failed to write to filter database %s. Wrote %zu out of %zu bytes\n",
                 qPrint(Doxygen::filterDBFileName),bytesWritten,bytesRead);
-            str.addChar('\0');
             Portable::pclose(f);
             fclose(bf);
             return false;
           }
           size+=bytesWritten;
-          str.addArray(buf,static_cast<uint>(bytesWritten));
+          str+=std::string_view(buf,bytesWritten);
         }
-        str.addChar('\0');
         item.fileSize = size;
         // add location entry to the dictionary
         m_cache.insert(std::make_pair(fileName.str(),item));
@@ -691,7 +706,7 @@ class FilterCache
 
     //! reads the fragment start at \a startLine and ending at \a endLine from file \a fileName
     //! into buffer \a str
-    bool getFileContentsDisk(const QCString &fileName,size_t startLine,size_t endLine,BufStr &str)
+    bool getFileContentsDisk(const QCString &fileName,size_t startLine,size_t endLine,std::string &str)
     {
       // normal file
       //printf("getFileContents(%s): no filter\n",qPrint(fileName));
@@ -715,14 +730,14 @@ class FilterCache
 
     //! computes the starting offset for each line for file \a fileName, whose contents should
     //! already be stored in buffer \a str.
-    void compileLineOffsets(const QCString &fileName,const BufStr &str)
+    void compileLineOffsets(const QCString &fileName,const std::string &str)
     {
-      const char *p=str.data();
       // line 1 (index 0) is at offset 0
       auto it = m_lineOffsets.insert(std::make_pair(fileName.data(),LineOffsets{0})).first;
+      const char *p=str.data();
       while (*p)
       {
-        char c;
+        char c=0;
         while ((c=*p)!='\n' && c!=0) p++; // search until end of the line
         p++;
         it->second.push_back(p-str.data());
@@ -730,19 +745,22 @@ class FilterCache
     }
 
     //! Returns the byte offset and size within a file of a fragment given the array of
-    //! line offsets and the start emd end line of the fragment.
+    //! line offsets and the start and end line of the fragment.
     auto getFragmentLocation(const LineOffsets &lineOffsets,
                              size_t startLine,size_t endLine) -> std::tuple<size_t,size_t>
     {
-      size_t startLineOffset = lineOffsets[std::min(startLine-1,lineOffsets.size()-1)];
-      size_t endLineOffset   = lineOffsets[std::min(endLine,    lineOffsets.size()-1)];
-      size_t fragmentSize = endLineOffset-startLineOffset;
+      assert(startLine > 0);
+      assert(startLine <= endLine);
+      const size_t startLineOffset = lineOffsets[std::min(startLine-1,lineOffsets.size()-1)];
+      const size_t endLineOffset   = lineOffsets[std::min(endLine,    lineOffsets.size()-1)];
+      assert(startLineOffset <= endLineOffset);
+      const size_t fragmentSize = endLineOffset-startLineOffset;
       return std::tie(startLineOffset,fragmentSize);
     };
 
     //! Shrinks buffer \a str which should hold the contents of \a fileName to the
     //! fragment starting a line \a startLine and ending at line \a endLine
-    void shrinkBuffer(BufStr &str,const QCString &fileName,size_t startLine,size_t endLine)
+    void shrinkBuffer(std::string &str,const QCString &fileName,size_t startLine,size_t endLine)
     {
       // compute offsets from start for each line
       compileLineOffsets(fileName,str);
@@ -752,23 +770,20 @@ class FilterCache
       auto [ startLineOffset, fragmentSize] = getFragmentLocation(lineOffsets,startLine,endLine);
       //printf("%s: new file [%zu-%zu]->[%zu-%zu] size=%zu\n",
       //    qPrint(fileName),startLine,endLine,startLineOffset,endLineOffset,fragmentSize);
-      str.dropFromStart(startLineOffset);
-      str.resize(fragmentSize+1);
-      str.at(fragmentSize)='\0';
+      str.erase(0,startLineOffset);
+      str.resize(fragmentSize);
     }
 
     //! Reads the fragment start at byte offset \a startOffset of file \a fileName into buffer \a str.
     //! Result will be a null terminated. If size==0 the whole file will be read and startOffset is ignored.
     //! If size>0, size bytes will be read.
-    void readFragmentFromFile(BufStr &str,const QCString &fileName,size_t startOffset,size_t size=0)
+    void readFragmentFromFile(std::string &str,const QCString &fileName,size_t startOffset,size_t size=0)
     {
       std::ifstream ifs = Portable::openInputStream(fileName,true,true);
       if (size==0) { startOffset=0; size = static_cast<size_t>(ifs.tellg()); }
       ifs.seekg(startOffset, std::ios::beg);
-      str.resize(size+1);
+      str.resize(size);
       ifs.read(str.data(), size);
-      str.skip(size);
-      str.addChar('\0');
     }
 
     FilterCache() : m_endPos(0) { }
@@ -798,7 +813,7 @@ FilterCache &FilterCache::instance()
  * The line actually containing the bracket is returned via endLine.
  * Note that for VHDL code the bracket search is not done.
  */
-bool readCodeFragment(const QCString &fileName,
+bool readCodeFragment(const QCString &fileName,bool isMacro,
                       int &startLine,int &endLine,QCString &result)
 {
   bool filterSourceFiles = Config_getBool(FILTER_SOURCE_FILES);
@@ -807,15 +822,16 @@ bool readCodeFragment(const QCString &fileName,
   int tabSize = Config_getInt(TAB_SIZE);
   SrcLangExt lang = getLanguageFromFileName(fileName);
   const int blockSize = 4096;
-  BufStr str(blockSize);
+  std::string str;
   FilterCache::instance().getFileContents(fileName,
-                                          static_cast<size_t>(startLine),
-                                          static_cast<size_t>(endLine),str);
+                                          static_cast<size_t>(std::max(1,startLine)),
+                                          static_cast<size_t>(std::max({1,startLine,endLine})),str);
   //printf("readCodeFragment(%s,startLine=%d,endLine=%d)=\n[[[\n%s]]]\n",qPrint(fileName),startLine,endLine,qPrint(str));
 
-  bool found = lang==SrcLangExt_VHDL   ||
-               lang==SrcLangExt_Python ||
-               lang==SrcLangExt_Fortran;
+  bool found = lang==SrcLangExt::VHDL   ||
+               lang==SrcLangExt::Python ||
+               lang==SrcLangExt::Fortran ||
+               isMacro;
                // for VHDL, Python, and Fortran no bracket search is possible
   char *p=str.data();
   if (p && *p)
@@ -892,7 +908,7 @@ bool readCodeFragment(const QCString &fileName,
       do
       {
         //printf("reading line %d in range %d-%d\n",lineNr,startLine,endLine);
-        int size_read;
+        int size_read=0;
         do
         {
           // read up to maxLineLength-1 bytes, the last byte being zero
@@ -914,7 +930,7 @@ bool readCodeFragment(const QCString &fileName,
       int braceIndex   = result.findRev('}');
       if (braceIndex > newLineIndex)
       {
-        result.truncate(static_cast<size_t>(braceIndex+1));
+        result.resize(static_cast<size_t>(braceIndex+1));
       }
       endLine=lineNr-1;
     }
@@ -924,7 +940,22 @@ bool readCodeFragment(const QCString &fileName,
       Debug::print(Debug::FilterOutput,0,"-------------\n%s\n-------------\n",qPrint(result));
     }
   }
-  result = transcodeCharacterStringToUTF8(getEncoding(FileInfo(fileName.str())),result);
+  QCString encoding = getEncoding(FileInfo(fileName.str()));
+  if (encoding!="UTF-8")
+  {
+    std::string encBuf = result.str();
+    bool ok = transcodeCharacterStringToUTF8(encBuf,encoding.data());
+    if (ok)
+    {
+      result = QCString(encBuf);
+    }
+    else
+    {
+      err("failed to transcode characters in code fragment in file %s lines %d to %d, from input encoding %s to UTF-8\n",
+          qPrint(fileName),startLine,endLine,qPrint(encoding));
+
+    }
+  }
   if (!result.isEmpty() && result.at(result.length()-1)!='\n') result += "\n";
   //printf("readCodeFragment(%d-%d)=%s\n",startLine,endLine,qPrint(result));
   return found;
@@ -1015,7 +1046,7 @@ void DefinitionImpl::writeSourceDef(OutputList &ol,const QCString &) const
 void DefinitionImpl::setBodySegment(int defLine, int bls,int ble)
 {
   //printf("setBodySegment(%d,%d) for %s\n",bls,ble,qPrint(name()));
-  if (m_impl->body==0) m_impl->body = new BodyInfo;
+  if (!m_impl->body) m_impl->body = make_DeepCopyUnique<BodyInfo>();
   m_impl->body->defLine   = defLine;
   m_impl->body->startLine = bls;
   m_impl->body->endLine   = ble;
@@ -1023,7 +1054,7 @@ void DefinitionImpl::setBodySegment(int defLine, int bls,int ble)
 
 void DefinitionImpl::setBodyDef(const FileDef *fd)
 {
-  if (m_impl->body==0) m_impl->body = new BodyInfo;
+  if (!m_impl->body) m_impl->body = make_DeepCopyUnique<BodyInfo>();
   m_impl->body->fileDef=fd;
 }
 
@@ -1037,28 +1068,29 @@ bool DefinitionImpl::hasSources() const
 /*! Write code of this definition into the documentation */
 void DefinitionImpl::writeInlineCode(OutputList &ol,const QCString &scopeName) const
 {
-  bool inlineSources = Config_getBool(INLINE_SOURCES);
-  ol.pushGeneratorState();
-  //printf("Source Fragment %s: %d-%d bodyDef=%p\n",qPrint(name()),
-  //        m_startBodyLine,m_endBodyLine,m_bodyDef);
+  const MemberDef *thisMd = nullptr;
+  if (m_impl->def->definitionType()==Definition::TypeMember)
+  {
+    thisMd = toMemberDef(m_impl->def);
+  }
+  bool inlineSources = thisMd && thisMd->hasInlineSource();
+  //printf("Source Fragment %s: %d-%d\n",qPrint(name()),
+  //        m_impl->body->startLine,m_impl->body->endLine);
   if (inlineSources && hasSources())
   {
+    ol.pushGeneratorState();
     QCString codeFragment;
+    bool isMacro = thisMd && thisMd->memberType()==MemberType_Define;
     int actualStart=m_impl->body->startLine,actualEnd=m_impl->body->endLine;
-    if (readCodeFragment(m_impl->body->fileDef->absFilePath(),
+    if (readCodeFragment(m_impl->body->fileDef->absFilePath(),isMacro,
           actualStart,actualEnd,codeFragment)
        )
     {
-      //printf("Adding code fragment '%s' ext='%s'\n",
-      //    qPrint(codeFragment),qPrint(m_impl->defFileExt));
+      //printf("Adding code fragment '%s' ext='%s' range=%d-%d\n",
+      //    qPrint(codeFragment),qPrint(m_impl->defFileExt),actualStart,actualEnd);
       auto intf = Doxygen::parserManager->getCodeParser(m_impl->defFileExt);
       intf->resetCodeParserState();
       //printf("Read:\n'%s'\n\n",qPrint(codeFragment));
-      const MemberDef *thisMd = 0;
-      if (m_impl->def->definitionType()==Definition::TypeMember)
-      {
-        thisMd = toMemberDef(m_impl->def);
-      }
 
       auto &codeOL = ol.codeGenerators();
       codeOL.startCodeFragment("DoxyCode");
@@ -1077,11 +1109,11 @@ void DefinitionImpl::writeInlineCode(OutputList &ol,const QCString &scopeName) c
                      );
       codeOL.endCodeFragment("DoxyCode");
     }
+    ol.popGeneratorState();
   }
-  ol.popGeneratorState();
 }
 
-static inline MemberVector refMapToVector(const std::unordered_map<std::string,const MemberDef *> &map)
+static inline MemberVector refMapToVector(const std::unordered_map<std::string,MemberDef *> &map)
 {
   // convert map to a vector of values
   MemberVector result;
@@ -1091,7 +1123,7 @@ static inline MemberVector refMapToVector(const std::unordered_map<std::string,c
                  { return item.second; }     // extract value to add from map Key,Value pair
                 );
   // and sort it
-  std::sort(result.begin(),result.end(),
+  std::stable_sort(result.begin(),result.end(),
               [](const auto &m1,const auto &m2) { return genericCompareMembers(m1,m2)<0; });
   return result;
 }
@@ -1100,7 +1132,7 @@ static inline MemberVector refMapToVector(const std::unordered_map<std::string,c
  *  definition is used.
  */
 void DefinitionImpl::_writeSourceRefList(OutputList &ol,const QCString &scopeName,
-    const QCString &text,const std::unordered_map<std::string,const MemberDef *> &membersMap,
+    const QCString &text,const std::unordered_map<std::string,MemberDef *> &membersMap,
     bool /*funcOnly*/) const
 {
   if (!membersMap.empty())
@@ -1212,7 +1244,7 @@ bool DefinitionImpl::hasUserDocumentation() const
 }
 
 
-void DefinitionImpl::addSourceReferencedBy(const MemberDef *md)
+void DefinitionImpl::addSourceReferencedBy(MemberDef *md)
 {
   if (md)
   {
@@ -1223,12 +1255,20 @@ void DefinitionImpl::addSourceReferencedBy(const MemberDef *md)
     {
       name.prepend(scope+"::");
     }
+    else if (md->isStatic() && md->getFileDef())
+    {
+      name.prepend(md->getFileDef()->name()+":");
+    }
+    if (md->isCallable())
+    {
+      name.append(md->argsString());
+    }
 
     m_impl->sourceRefByDict.insert({name.str(),md});
   }
 }
 
-void DefinitionImpl::addSourceReferences(const MemberDef *md)
+void DefinitionImpl::addSourceReferences(MemberDef *md)
 {
   if (md)
   {
@@ -1239,6 +1279,14 @@ void DefinitionImpl::addSourceReferences(const MemberDef *md)
     {
       name.prepend(scope+"::");
     }
+    else if (md->isStatic() && md->getFileDef())
+    {
+      name.prepend(md->getFileDef()->name()+":");
+    }
+    if (md->isCallable())
+    {
+      name.append(md->argsString());
+    }
 
     m_impl->sourceRefsDict.insert({name.str(),md});
   }
@@ -1246,10 +1294,10 @@ void DefinitionImpl::addSourceReferences(const MemberDef *md)
 
 const Definition *DefinitionImpl::findInnerCompound(const QCString &) const
 {
-  return 0;
+  return nullptr;
 }
 
-void DefinitionImpl::addInnerCompound(const Definition *)
+void DefinitionImpl::addInnerCompound(Definition *)
 {
   err("DefinitionImpl::addInnerCompound() called\n");
 }
@@ -1265,7 +1313,7 @@ QCString DefinitionImpl::qualifiedName() const
   }
 
   //printf("start %s::qualifiedName() localName=%s\n",qPrint(name()),qPrint(m_impl->localName));
-  if (m_impl->outerScope==0)
+  if (m_impl->outerScope==nullptr)
   {
     if (m_impl->localName=="<globalScope>")
     {
@@ -1306,18 +1354,18 @@ void DefinitionImpl::setOuterScope(Definition *d)
   }
   if (!found)
   {
-    m_impl->qualifiedName.resize(0); // flush cached scope name
+    m_impl->qualifiedName.clear(); // flush cached scope name
     m_impl->outerScope = d;
   }
   m_impl->hidden = m_impl->hidden || d->isHidden();
 }
 
-QCString DefinitionImpl::localName() const
+const QCString &DefinitionImpl::localName() const
 {
   return m_impl->localName;
 }
 
-void DefinitionImpl::makePartOfGroup(const GroupDef *gd)
+void DefinitionImpl::makePartOfGroup(GroupDef *gd)
 {
   m_impl->partOfGroups.push_back(gd);
 }
@@ -1337,7 +1385,7 @@ void DefinitionImpl::mergeRefItems(Definition *d)
                                 otherXrefList.begin(),otherXrefList.end());
 
   // sort results on itemId
-  std::sort(m_impl->xrefListItems.begin(),m_impl->xrefListItems.end(),
+  std::stable_sort(m_impl->xrefListItems.begin(),m_impl->xrefListItems.end(),
             [](RefItem *left,RefItem *right)
             { return  left->id() <right->id() ||
                      (left->id()==right->id() &&
@@ -1425,16 +1473,18 @@ QCString DefinitionImpl::navigationPathAsString() const
   result+="<li class=\"navelem\">";
   if (m_impl->def->isLinkableInProject())
   {
+    QCString fn = m_impl->def->getOutputFileBase();
+    addHtmlExtensionIfMissing(fn);
     if (m_impl->def->definitionType()==Definition::TypeGroup &&
         !toGroupDef(m_impl->def)->groupTitle().isEmpty())
     {
-      result+="<a class=\"el\" href=\"$relpath^"+addHtmlExtensionIfMissing(m_impl->def->getOutputFileBase())+"\">"+
+      result+="<a class=\"el\" href=\"$relpath^"+fn+"\">"+
               convertToHtml(toGroupDef(m_impl->def)->groupTitle())+"</a>";
     }
     else if (m_impl->def->definitionType()==Definition::TypePage &&
              toPageDef(m_impl->def)->hasTitle())
     {
-      result+="<a class=\"el\" href=\"$relpath^"+addHtmlExtensionIfMissing(m_impl->def->getOutputFileBase())+"\">"+
+      result+="<a class=\"el\" href=\"$relpath^"+fn+"\">"+
             convertToHtml((toPageDef(m_impl->def))->title())+"</a>";
     }
     else if (m_impl->def->definitionType()==Definition::TypeClass)
@@ -1444,13 +1494,13 @@ QCString DefinitionImpl::navigationPathAsString() const
       {
         name = name.left(name.length()-2);
       }
-      result+="<a class=\"el\" href=\"$relpath^"+addHtmlExtensionIfMissing(m_impl->def->getOutputFileBase());
+      result+="<a class=\"el\" href=\"$relpath^"+fn;
       if (!m_impl->def->anchor().isEmpty()) result+="#"+m_impl->def->anchor();
       result+="\">"+convertToHtml(name)+"</a>";
     }
     else
     {
-      result+="<a class=\"el\" href=\"$relpath^"+addHtmlExtensionIfMissing(m_impl->def->getOutputFileBase())+"\">"+
+      result+="<a class=\"el\" href=\"$relpath^"+fn+"\">"+
               convertToHtml(locName)+"</a>";
     }
   }
@@ -1479,144 +1529,10 @@ void DefinitionImpl::writeNavigationPath(OutputList &ol) const
   ol.popGeneratorState();
 }
 
-// TODO: move to htmlgen
 void DefinitionImpl::writeToc(OutputList &ol, const LocalToc &localToc) const
 {
   if (m_impl->sectionRefs.empty()) return;
-  if (localToc.isHtmlEnabled())
-  {
-    int maxLevel = localToc.htmlLevel();
-    ol.pushGeneratorState();
-    ol.disableAllBut(OutputType::Html);
-    ol.writeString("<div class=\"toc\">");
-    ol.writeString("<h3>");
-    ol.writeString(theTranslator->trRTFTableOfContents());
-    ol.writeString("</h3>\n");
-    ol.writeString("<ul>");
-    int level=1,l;
-    char cs[2];
-    cs[1]='\0';
-    BoolVector inLi(maxLevel+1,false);
-    for (const SectionInfo *si : m_impl->sectionRefs)
-    {
-      SectionType type = si->type();
-      if (isSection(type))
-      {
-        //printf("  level=%d title=%s\n",level,qPrint(si->title));
-        int nextLevel = static_cast<int>(type);
-        if (nextLevel>level)
-        {
-          for (l=level;l<nextLevel;l++)
-          {
-            if (l < maxLevel) ol.writeString("<ul>");
-          }
-        }
-        else if (nextLevel<level)
-        {
-          for (l=level;l>nextLevel;l--)
-          {
-            if (l <= maxLevel && inLi[l]) ol.writeString("</li>\n");
-            inLi[l]=false;
-            if (l <= maxLevel) ol.writeString("</ul>\n");
-          }
-        }
-        cs[0]=static_cast<char>('0'+nextLevel);
-        if (nextLevel <= maxLevel && inLi[nextLevel])
-        {
-          ol.writeString("</li>\n");
-        }
-        QCString titleDoc = convertToHtml(si->title());
-        if (nextLevel <= maxLevel)
-        {
-          ol.writeString("<li class=\"level"+QCString(cs)+"\">"
-                         "<a href=\"#"+si->label()+"\">"+
-                         (si->title().isEmpty()?si->label():titleDoc)+"</a>");
-        }
-        inLi[nextLevel]=true;
-        level = nextLevel;
-      }
-    }
-    if (level > maxLevel) level = maxLevel;
-    while (level>1 && level <= maxLevel)
-    {
-      if (inLi[level])
-      {
-        ol.writeString("</li>\n");
-      }
-      inLi[level]=FALSE;
-      ol.writeString("</ul>\n");
-      level--;
-    }
-    if (level <= maxLevel && inLi[level]) ol.writeString("</li>\n");
-    ol.writeString("</ul>\n");
-    ol.writeString("</div>\n");
-    ol.popGeneratorState();
-  }
-
-  if (localToc.isDocbookEnabled())
-  {
-    ol.pushGeneratorState();
-    ol.disableAllBut(OutputType::Docbook);
-    ol.writeString("    <toc>\n");
-    ol.writeString("    <title>" + theTranslator->trRTFTableOfContents() + "</title>\n");
-    int level=1,l;
-    int maxLevel = localToc.docbookLevel();
-    BoolVector inLi(maxLevel+1,false);
-    for (const SectionInfo *si : m_impl->sectionRefs)
-    {
-      SectionType type = si->type();
-      if (isSection(type))
-      {
-        //printf("  level=%d title=%s\n",level,qPrint(si->title));
-        int nextLevel = static_cast<int>(type);
-        if (nextLevel>level)
-        {
-          for (l=level;l<nextLevel;l++)
-          {
-            if (l < maxLevel) ol.writeString("    <tocdiv>\n");
-          }
-        }
-        else if (nextLevel<level)
-        {
-          for (l=level;l>nextLevel;l--)
-          {
-            inLi[l]=FALSE;
-            if (l <= maxLevel) ol.writeString("    </tocdiv>\n");
-          }
-        }
-        if (nextLevel <= maxLevel)
-        {
-          QCString titleDoc = convertToDocBook(si->title());
-          ol.writeString("      <tocentry>" +
-              (si->title().isEmpty()?si->label():titleDoc) +
-              "</tocentry>\n");
-        }
-        inLi[nextLevel]=TRUE;
-        level = nextLevel;
-      }
-    }
-    if (level > maxLevel) level = maxLevel;
-    while (level>1 && level <= maxLevel)
-    {
-      inLi[level]=FALSE;
-      ol.writeString("</tocdiv>\n");
-      level--;
-    }
-    ol.writeString("    </toc>\n");
-    ol.popGeneratorState();
-  }
-
-  if (localToc.isLatexEnabled())
-  {
-    ol.pushGeneratorState();
-    ol.disableAllBut(OutputType::Latex);
-    int maxLevel = localToc.latexLevel();
-
-    ol.writeString("\\etocsetnexttocdepth{"+QCString().setNum(maxLevel)+"}\n");
-
-    ol.writeString("\\localtableofcontents\n");
-    ol.popGeneratorState();
-  }
+  ol.writeLocalToc(m_impl->sectionRefs,localToc);
 }
 
 //----------------------------------------------------------------------------------------
@@ -1708,7 +1624,7 @@ void DefinitionImpl::computeTooltip()
 {
   if (m_impl->brief && m_impl->brief->tooltip.isEmpty() && !m_impl->brief->doc.isEmpty())
   {
-    const MemberDef *md = m_impl->def->definitionType()==Definition::TypeMember ? toMemberDef(m_impl->def) : 0;
+    const MemberDef *md = m_impl->def->definitionType()==Definition::TypeMember ? toMemberDef(m_impl->def) : nullptr;
     const Definition *scope = m_impl->def->definitionType()==Definition::TypeMember ? m_impl->def->getOuterScope() : m_impl->def;
     m_impl->brief->tooltip = parseCommentAsText(scope,md,
                                 m_impl->brief->doc, m_impl->brief->file, m_impl->brief->line);
@@ -1780,6 +1696,11 @@ bool DefinitionImpl::isArtificial() const
   return m_impl->isArtificial;
 }
 
+bool DefinitionImpl::isExported() const
+{
+  return m_impl->isExported;
+}
+
 QCString DefinitionImpl::getReference() const
 {
   return m_impl->ref;
@@ -1807,7 +1728,7 @@ int DefinitionImpl::getEndBodyLine() const
 
 const FileDef *DefinitionImpl::getBodyDef() const
 {
-  return m_impl->body ? m_impl->body->fileDef : 0;
+  return m_impl->body ? m_impl->body->fileDef : nullptr;
 }
 
 const GroupList &DefinitionImpl::partOfGroups() const
@@ -1904,6 +1825,11 @@ void DefinitionImpl::setArtificial(bool b)
   m_impl->isArtificial = b;
 }
 
+void DefinitionImpl::setExported(bool b)
+{
+  m_impl->isExported = b;
+}
+
 void DefinitionImpl::setLocalName(const QCString &name)
 {
   m_impl->localName=name;
@@ -1940,7 +1866,7 @@ QCString DefinitionImpl::externalReference(const QCString &relPath) const
     if (it!=Doxygen::tagDestinationMap.end())
     {
       QCString result(it->second);
-      uint l = result.length();
+      size_t l = result.length();
       if (!relPath.isEmpty() && l>0 && result.at(0)=='.')
       { // relative path -> prepend relPath.
         result.prepend(relPath);
@@ -1953,7 +1879,7 @@ QCString DefinitionImpl::externalReference(const QCString &relPath) const
   return relPath;
 }
 
-QCString DefinitionImpl::name() const
+const QCString &DefinitionImpl::name() const
 {
   return m_impl->name;
 }
@@ -2003,42 +1929,48 @@ void DefinitionAliasImpl::deinit()
   removeFromMap(m_symbolName,m_def);
 }
 
-QCString DefinitionAliasImpl::qualifiedName() const
+void DefinitionAliasImpl::updateQualifiedName() const
 {
-  //printf("start %s::qualifiedName() localName=%s\n",qPrint(name()),qPrint(m_impl->localName));
-  if (m_scope==0)
+  std::lock_guard<std::recursive_mutex> lock(g_qualifiedNameMutex);
+  if (m_qualifiedName.isEmpty())
   {
-    return m_def->localName();
-  }
-  else
-  {
-    return m_scope->qualifiedName()+
-           getLanguageSpecificSeparator(m_scope->getLanguage())+
-           m_def->localName();
+    //printf("start %s::qualifiedName() localName=%s\n",qPrint(name()),qPrint(m_impl->localName));
+    if (m_scope==nullptr)
+    {
+      m_qualifiedName = m_def->localName();
+    }
+    else
+    {
+      m_qualifiedName = m_scope->qualifiedName()+
+        getLanguageSpecificSeparator(m_scope->getLanguage())+
+        m_def->localName();
+    }
   }
 }
 
-QCString DefinitionAliasImpl::name() const
+QCString DefinitionAliasImpl::qualifiedName() const
 {
-  return qualifiedName();
+  updateQualifiedName();
+  return m_qualifiedName;
+}
+
+const QCString &DefinitionAliasImpl::name() const
+{
+  updateQualifiedName();
+  return m_qualifiedName;
 }
 
 //---------------------------------------------------------------------------------
 
 Definition *toDefinition(DefinitionMutable *dm)
 {
-  if (dm==0) return 0;
+  if (dm==nullptr) return nullptr;
   return dm->toDefinition_();
 }
 
 DefinitionMutable *toDefinitionMutable(Definition *d)
 {
-  if (d==0) return 0;
+  if (d==nullptr) return nullptr;
   return d->toDefinitionMutable_();
 }
 
-DefinitionMutable *toDefinitionMutable(const Definition *d)
-{
-  return toDefinitionMutable(const_cast<Definition*>(d));
-}
-
