@@ -30,7 +30,7 @@
 #include "docparser.h"
 #include "definitionimpl.h"
 #include "filedef.h"
-
+#include "trace.h"
 
 //----------------------------------------------------------------------
 
@@ -38,39 +38,43 @@ class DirDefImpl : public DefinitionMixin<DirDef>
 {
   public:
     DirDefImpl(const QCString &path);
-    virtual ~DirDefImpl();
+   ~DirDefImpl() override;
+    NON_COPYABLE(DirDefImpl)
 
-    virtual DefType definitionType() const { return TypeDir; }
-    virtual CodeSymbolType codeSymbolType() const { return CodeSymbolType::Default; }
-    virtual QCString getOutputFileBase() const;
-    virtual QCString anchor() const { return QCString(); }
-    virtual bool isLinkableInProject() const;
-    virtual bool isLinkable() const;
-    virtual QCString displayName(bool=TRUE) const { return m_dispName; }
-    virtual const QCString shortName() const { return m_shortName; }
-    virtual void addSubDir(DirDef *subdir);
-    virtual const FileList &getFiles() const { return m_fileList; }
-    virtual void addFile(const FileDef *fd);
-    virtual const DirList &subDirs() const { return m_subdirs; }
-    virtual bool hasSubdirs() const { return !m_subdirs.empty(); }
-    virtual int level() const { return m_level; }
-    virtual DirDef *parent() const { return m_parent; }
-    virtual int dirCount() const { return m_dirCount; }
-    virtual const UsedDirLinkedMap &usedDirs() const { return m_usedDirs; }
-    virtual bool isParentOf(const DirDef *dir) const;
-    virtual bool depGraphIsTrivial() const;
-    virtual QCString shortTitle() const;
-    virtual bool hasDetailedDescription() const;
-    virtual void writeDocumentation(OutputList &ol);
-    virtual void writeTagFile(TextStream &t);
-    virtual void setDiskName(const QCString &name) { m_diskName = name; }
-    virtual void sort();
-    virtual void setParent(DirDef *parent);
-    virtual void setDirCount(int count);
-    virtual void setLevel();
-    virtual void addUsesDependency(const DirDef *usedDir,const FileDef *srcFd,
-                                   const FileDef *dstFd,bool srcDirect, bool dstDirect);
-    virtual void computeDependencies();
+    DefType definitionType() const override { return TypeDir; }
+    CodeSymbolType codeSymbolType() const override { return CodeSymbolType::Default; }
+    QCString getOutputFileBase() const override;
+    QCString anchor() const override { return QCString(); }
+    bool isLinkableInProject() const override;
+    bool isLinkable() const override;
+    QCString displayName(bool=TRUE) const override { return m_dispName; }
+    const QCString shortName() const override { return m_shortName; }
+    void addSubDir(DirDef *subdir) override;
+    const FileList &getFiles() const override { return m_fileList; }
+    void addFile(const FileDef *fd) override;
+    const DirList &subDirs() const override { return m_subdirs; }
+    bool hasSubdirs() const override { return !m_subdirs.empty(); }
+    int level() const override { return m_level; }
+    DirDef *parent() const override { return m_parent; }
+    int dirIndex() const override { return m_dirIndex; }
+    const UsedDirLinkedMap &usedDirs() const override { return m_usedDirs; }
+    bool isParentOf(const DirDef *dir) const override;
+    bool depGraphIsTrivial() const override;
+    QCString shortTitle() const override;
+    bool hasDetailedDescription() const override;
+    void writeDocumentation(OutputList &ol) override;
+    void writeTagFile(TextStream &t) override;
+    void setDiskName(const QCString &name) override { m_diskName = name; }
+    void sort() override;
+    void setParent(DirDef *parent) override;
+    void setDirIndex(int index) override;
+    void setLevel() override;
+    void addUsesDependency(const DirDef *usedDir,const FileDef *srcFd,
+                                   const FileDef *dstFd,bool srcDirect, bool dstDirect) override;
+    void computeDependencies() override;
+
+    bool hasDirectoryGraph() const override;
+    void overrideDirectoryGraph(bool e) override;
 
   public:
     static DirDef *mergeDirectoryInTree(const QCString &path);
@@ -93,10 +97,11 @@ class DirDefImpl : public DefinitionMixin<DirDef>
     QCString m_shortName;
     QCString m_diskName;
     FileList m_fileList;                 // list of files in the group
-    int m_dirCount = -1;
+    int m_dirIndex = -1;
     int m_level;
     DirDef *m_parent;
     UsedDirLinkedMap m_usedDirs;
+    bool m_hasDirectoryGraph = false;
 };
 
 DirDef *createDirDef(const QCString &path)
@@ -132,7 +137,9 @@ DirDefImpl::DirDefImpl(const QCString &path) : DefinitionMixin(path,1,1,path)
   }
 
   m_level=-1;
-  m_parent=0;
+  m_parent=nullptr;
+  m_hasDirectoryGraph=Config_getBool(DIRECTORY_GRAPH);
+
 }
 
 DirDefImpl::~DirDefImpl()
@@ -161,9 +168,9 @@ void DirDefImpl::setParent(DirDef *p)
    m_parent=p;
 }
 
-void DirDefImpl::setDirCount(int count)
+void DirDefImpl::setDirIndex(int index)
 {
-  m_dirCount=count;
+  m_dirIndex=index;
 }
 
 void DirDefImpl::addFile(const FileDef *fd)
@@ -174,17 +181,19 @@ void DirDefImpl::addFile(const FileDef *fd)
 
 void DirDefImpl::sort()
 {
-  std::sort(m_subdirs.begin(), m_subdirs.end(), compareDirDefs);
-  std::sort(m_fileList.begin(), m_fileList.end(), compareFileDefs);
+  std::stable_sort(m_subdirs.begin(), m_subdirs.end(), compareDirDefs);
+  std::stable_sort(m_fileList.begin(), m_fileList.end(), compareFileDefs);
 }
 
 static QCString encodeDirName(const QCString &anchor)
 {
+  AUTO_TRACE();
   // convert to md5 hash
-  uchar md5_sig[16];
+  uint8_t md5_sig[16];
   char sigStr[33];
-  MD5Buffer(anchor.data(),anchor.length(),md5_sig);
+  MD5Buffer(anchor.data(),static_cast<unsigned int>(anchor.length()),md5_sig);
   MD5SigToString(md5_sig,sigStr);
+  AUTO_TRACE_EXIT("result={}",sigStr);
   return sigStr;
 
   // old algorithm
@@ -212,13 +221,14 @@ static QCString encodeDirName(const QCString &anchor)
 
 QCString DirDefImpl::getOutputFileBase() const
 {
-  //printf("DirDefImpl::getOutputFileBase() %s->dir_%s\n",
-  //    qPrint(m_diskName),qPrint(encodeDirName(m_diskName)));
-  return "dir_"+encodeDirName(m_diskName);
+  QCString dir = "dir_"+encodeDirName(m_diskName);
+  AUTO_TRACE("diskName={} result={}",m_diskName,dir);
+  return dir;
 }
 
 void DirDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title)
 {
+  AUTO_TRACE();
   if ((!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF)) ||
       !documentation().isEmpty())
   {
@@ -237,7 +247,7 @@ void DirDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title)
     // repeat brief description
     if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF))
     {
-      ol.generateDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE,
+      ol.generateDoc(briefFile(),briefLine(),this,nullptr,briefDescription(),FALSE,FALSE,
                      QCString(),FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
     }
     // separator between brief and details
@@ -247,7 +257,6 @@ void DirDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title)
       ol.pushGeneratorState();
         ol.disable(OutputType::Man);
         ol.disable(OutputType::RTF);
-        // ol.newParagraph();  // FIXME:PARA
         ol.enableAll();
         ol.disableAllBut(OutputType::Man);
         ol.enable(OutputType::Latex);
@@ -258,7 +267,7 @@ void DirDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title)
     // write documentation
     if (!documentation().isEmpty())
     {
-      ol.generateDoc(docFile(),docLine(),this,0,documentation()+"\n",TRUE,FALSE,
+      ol.generateDoc(docFile(),docLine(),this,nullptr,documentation()+"\n",TRUE,FALSE,
                      QCString(),FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
     }
   }
@@ -266,11 +275,12 @@ void DirDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title)
 
 void DirDefImpl::writeBriefDescription(OutputList &ol)
 {
+  AUTO_TRACE();
   if (hasBriefDescription())
   {
     auto parser { createDocParser() };
     auto ast    { validatingParseDoc(
-         *parser.get(), briefFile(),briefLine(),this,0,briefDescription(),TRUE,FALSE,
+         *parser.get(), briefFile(),briefLine(),this,nullptr,briefDescription(),TRUE,FALSE,
          QCString(),FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT)) };
     if (!ast->isEmpty())
     {
@@ -279,7 +289,7 @@ void DirDefImpl::writeBriefDescription(OutputList &ol)
       ol.disableAllBut(OutputType::Man);
       ol.writeString(" - ");
       ol.popGeneratorState();
-      ol.writeDoc(ast.get(),this,0);
+      ol.writeDoc(ast.get(),this,nullptr);
       ol.pushGeneratorState();
       ol.disable(OutputType::RTF);
       ol.writeString(" \n");
@@ -305,7 +315,7 @@ void DirDefImpl::writeBriefDescription(OutputList &ol)
 void DirDefImpl::writeDirectoryGraph(OutputList &ol)
 {
   // write graph dependency graph
-  if (Config_getBool(DIRECTORY_GRAPH) && Config_getBool(HAVE_DOT))
+  if (Config_getBool(HAVE_DOT) && m_hasDirectoryGraph /*&& Config_getBool(DIRECTORY_GRAPH)*/)
   {
     DotDirDeps dirDep(this);
     if (!dirDep.isTrivial())
@@ -324,6 +334,7 @@ void DirDefImpl::writeDirectoryGraph(OutputList &ol)
 
 void DirDefImpl::writeSubDirList(OutputList &ol)
 {
+  AUTO_TRACE();
   int numSubdirs = 0;
   for(const auto dd : m_subdirs)
   {
@@ -333,6 +344,7 @@ void DirDefImpl::writeSubDirList(OutputList &ol)
     }
   }
 
+  AUTO_TRACE_ADD("numSubdirs={}",numSubdirs);
   // write subdir list
   if (numSubdirs>0)
   {
@@ -345,15 +357,23 @@ void DirDefImpl::writeSubDirList(OutputList &ol)
       if (dd->hasDocumentation() || !dd->getFiles().empty())
       {
         ol.startMemberDeclaration();
-        ol.startMemberItem(dd->anchor(),0);
-        ol.parseText(theTranslator->trDir(FALSE,TRUE)+" ");
+        ol.startMemberItem(dd->anchor(),OutputGenerator::MemberItemType::Normal);
+        {
+          ol.pushGeneratorState();
+          ol.disableAllBut(OutputType::Html);
+          ol.writeString("<span class=\"iconfclosed\"></span>");
+          ol.enableAll();
+          ol.disable(OutputType::Html);
+          ol.parseText(theTranslator->trDir(FALSE,TRUE)+" ");
+          ol.popGeneratorState();
+        }
         ol.insertMemberAlign();
         ol.writeObjectLink(dd->getReference(),dd->getOutputFileBase(),QCString(),dd->shortName());
-        ol.endMemberItem();
+        ol.endMemberItem(OutputGenerator::MemberItemType::Normal);
         if (!dd->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
         {
           ol.startMemberDescription(dd->getOutputFileBase());
-          ol.generateDoc(briefFile(),briefLine(),dd,0,dd->briefDescription(),
+          ol.generateDoc(briefFile(),briefLine(),dd,nullptr,dd->briefDescription(),
               FALSE, // indexWords
               FALSE, // isExample
               QCString(), // exampleName
@@ -373,10 +393,11 @@ void DirDefImpl::writeSubDirList(OutputList &ol)
 
 void DirDefImpl::writeFileList(OutputList &ol)
 {
+  AUTO_TRACE();
   int numFiles = 0;
   for (const auto &fd : m_fileList)
   {
-    bool genSourceFile;
+    bool genSourceFile=false;
     if (fileVisibleInIndex(fd,genSourceFile))
     {
       numFiles++;
@@ -387,6 +408,7 @@ void DirDefImpl::writeFileList(OutputList &ol)
     }
   }
 
+  AUTO_TRACE_ADD("numFiles={}",numFiles);
   // write file list
   if (numFiles>0)
   {
@@ -396,41 +418,46 @@ void DirDefImpl::writeFileList(OutputList &ol)
     ol.startMemberList();
     for (const auto &fd : m_fileList)
     {
-      bool doc,src;
-      doc = fileVisibleInIndex(fd,src);
+      bool src = false;
+      bool doc = fileVisibleInIndex(fd,src);
       if (doc || src)
       {
         ol.startMemberDeclaration();
-        ol.startMemberItem(fd->anchor(),0);
-        ol.docify(theTranslator->trFile(FALSE,TRUE)+" ");
+        ol.startMemberItem(fd->anchor(),OutputGenerator::MemberItemType::Normal);
+        {
+          ol.pushGeneratorState();
+          ol.disableAllBut(OutputType::Html);
+          bool genSrc = fd->generateSourceFile();
+          if (genSrc)
+          {
+            ol.startTextLink(fd->includeName(),QCString());
+          }
+          ol.writeString("<span class=\"icondoc\"></span>");
+          if (genSrc)
+          {
+            ol.endTextLink();
+          }
+          ol.enableAll();
+          ol.disable(OutputType::Html);
+          ol.docify(theTranslator->trFile(FALSE,TRUE)+" ");
+          ol.popGeneratorState();
+        }
         ol.insertMemberAlign();
         if (fd->isLinkable())
         {
-          ol.writeObjectLink(fd->getReference(),fd->getOutputFileBase(),QCString(),fd->name());
+          ol.writeObjectLink(fd->getReference(),fd->getOutputFileBase(),QCString(),fd->displayName());
         }
         else
         {
           ol.startBold();
-          ol.docify(fd->name());
+          ol.docify(fd->displayName());
           ol.endBold();
         }
-        if (fd->generateSourceFile())
-        {
-          ol.pushGeneratorState();
-          ol.disableAllBut(OutputType::Html);
-          ol.docify(" ");
-          ol.startTextLink(fd->includeName(),QCString());
-          ol.docify("[");
-          ol.parseText(theTranslator->trCode());
-          ol.docify("]");
-          ol.endTextLink();
-          ol.popGeneratorState();
-        }
-        ol.endMemberItem();
+        ol.endMemberItem(OutputGenerator::MemberItemType::Normal);
         if (!fd->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
         {
           ol.startMemberDescription(fd->getOutputFileBase());
-          ol.generateDoc(briefFile(),briefLine(),fd,0,fd->briefDescription(),
+          ol.generateDoc(briefFile(),briefLine(),fd,nullptr,fd->briefDescription(),
               FALSE, // indexWords
               FALSE, // isExample
               QCString(), // exampleName
@@ -473,7 +500,9 @@ void DirDefImpl::writeTagFile(TextStream &tagFile)
   tagFile << "  <compound kind=\"dir\">\n";
   tagFile << "    <name>" << convertToXML(displayName()) << "</name>\n";
   tagFile << "    <path>" << convertToXML(stripFromPath(name())) << "</path>\n";
-  tagFile << "    <filename>" << addHtmlExtensionIfMissing(getOutputFileBase()) << "</filename>\n";
+  QCString fn=getOutputFileBase();
+  addHtmlExtensionIfMissing(fn);
+  tagFile << "    <filename>" << fn << "</filename>\n";
   for (const auto &lde : LayoutDocManager::instance().docEntries(LayoutDocManager::Directory))
   {
     switch (lde->kind())
@@ -511,6 +540,7 @@ void DirDefImpl::writeDocumentation(OutputList &ol)
   ol.pushGeneratorState();
 
   QCString title=theTranslator->trDirReference(m_dispName);
+  AUTO_TRACE("title={}",title);
   startFile(ol,getOutputFileBase(),name(),title,HighlightedItem::Files,!generateTreeView);
 
   if (!generateTreeView)
@@ -595,6 +625,7 @@ void DirDefImpl::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::FileInlineClasses:
       case LayoutDocEntry::GroupClasses:
       case LayoutDocEntry::GroupConcepts:
+      case LayoutDocEntry::GroupModules:
       case LayoutDocEntry::GroupInlineClasses:
       case LayoutDocEntry::GroupNamespaces:
       case LayoutDocEntry::GroupDirs:
@@ -602,6 +633,10 @@ void DirDefImpl::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::GroupFiles:
       case LayoutDocEntry::GroupGraph:
       case LayoutDocEntry::GroupPageDocs:
+      case LayoutDocEntry::ModuleExports:
+      case LayoutDocEntry::ModuleClasses:
+      case LayoutDocEntry::ModuleConcepts:
+      case LayoutDocEntry::ModuleUsedFiles:
       case LayoutDocEntry::AuthorSection:
       case LayoutDocEntry::MemberGroups:
       case LayoutDocEntry::MemberDecl:
@@ -649,12 +684,11 @@ void DirDefImpl::addUsesDependency(const DirDef *dir,const FileDef *srcFd,
                                    const FileDef *dstFd,bool srcDirect, bool dstDirect)
 {
   if (this==dir) return; // do not add self-dependencies
-  //static int count=0;
-  //printf("  %d add dependency %s->%s due to %s->%s\n",
-  //    count++,qPrint(shortName()),
-  //    qPrint(dir->shortName()),
-  //    qPrint(srcFd->name()),
-  //    qPrint(dstFd->name()));
+  AUTO_TRACE("add dependency {}->{} due to {}->{}",
+      qPrint(shortName()),
+      qPrint(dir->shortName()),
+      qPrint(srcFd->name()),
+      qPrint(dstFd->name()));
 
   // levels match => add direct dependency
   bool added=FALSE;
@@ -662,9 +696,9 @@ void DirDefImpl::addUsesDependency(const DirDef *dir,const FileDef *srcFd,
   if (usedDir) // dir dependency already present
   {
      const FilePair *usedPair = usedDir->findFilePair(FilePair::key(srcFd,dstFd));
-     if (usedPair==0) // new file dependency
+     if (usedPair==nullptr) // new file dependency
      {
-       //printf("  => new file\n");
+       AUTO_TRACE_ADD("{} => {} new file dependency",srcFd->name(),dstFd->name());
        usedDir->addFileDep(srcFd,dstFd, srcDirect, dstDirect);
        added=TRUE;
      }
@@ -675,7 +709,7 @@ void DirDefImpl::addUsesDependency(const DirDef *dir,const FileDef *srcFd,
   }
   else // new directory dependency
   {
-    //printf("  => new file\n");
+    AUTO_TRACE_ADD("{} => {} new file dependency",srcFd->name(),dstFd->name());
     auto newUsedDir = std::make_unique<UsedDir>(dir);
     newUsedDir->addFileDep(srcFd,dstFd, srcDirect, dstDirect);
     m_usedDirs.add(dir->getOutputFileBase(),std::move(newUsedDir));
@@ -708,31 +742,29 @@ void DirDefImpl::addUsesDependency(const DirDef *dir,const FileDef *srcFd,
  */
 void DirDefImpl::computeDependencies()
 {
+  AUTO_TRACE();
   for (const auto &fd : m_fileList)
   {
-    //printf("  File %s\n",qPrint(fd->name()));
-    //printf("** dir=%s file=%s\n",qPrint(shortName()),qPrint(fd->name()));
+    AUTO_TRACE_ADD("dir={} file={}",shortName(),fd->name());
     for (const auto &ii : fd->includeFileList())
     {
-      //printf("  > %s\n",qPrint(ii->includeName));
-      //printf("    #include %s\n",qPrint(ii->includeName));
+      AUTO_TRACE_ADD("#include {}",ii.includeName);
       if (ii.fileDef && ii.fileDef->isLinkable()) // linkable file
       {
         DirDef *usedDir = ii.fileDef->getDirDef();
         if (usedDir)
         {
           // add dependency: thisDir->usedDir
-          //static int count=0;
-          //printf("      %d: add dependency %s->%s\n",count++,qPrint(name()),qPrint(usedDir->name()));
+          AUTO_TRACE_ADD("add dependency {}->{}",name(),usedDir->name());
           addUsesDependency(usedDir,fd,ii.fileDef,true,true);
         }
       }
     }
   }
 
-  std::sort(m_usedDirs.begin(),m_usedDirs.end(),
+  std::stable_sort(m_usedDirs.begin(),m_usedDirs.end(),
             [](const auto &u1,const auto &u2)
-            { return qstricmp(u1->dir()->getOutputFileBase(),u2->dir()->getOutputFileBase())<0; });
+            { return qstricmp_sort(u1->dir()->getOutputFileBase(),u2->dir()->getOutputFileBase())<0; });
 
   for (const auto& usedDirectory : m_usedDirs)
   {
@@ -755,7 +787,6 @@ bool DirDefImpl::depGraphIsTrivial() const
   return m_usedDirs.empty() && m_parent==nullptr;
 }
 
-
 //----------------------------------------------------------------------
 
 UsedDir::UsedDir(const DirDef *dir) :
@@ -763,10 +794,6 @@ UsedDir::UsedDir(const DirDef *dir) :
 {
 }
 
-UsedDir::~UsedDir()
-{
-}
-
 void UsedDir::addFileDep(const FileDef *srcFd,const FileDef *dstFd, bool srcDirect, bool dstDirect)
 {
   m_filePairs.add(FilePair::key(srcFd,dstFd),std::make_unique<FilePair>(srcFd,dstFd));
@@ -777,13 +804,13 @@ void UsedDir::addFileDep(const FileDef *srcFd,const FileDef *dstFd, bool srcDire
 
 void UsedDir::sort()
 {
-  std::sort(m_filePairs.begin(),
+  std::stable_sort(m_filePairs.begin(),
             m_filePairs.end(),
             [](const auto &left,const auto &right)
             {
-              int orderHi = qstricmp(left->source()->name(),right->source()->name());
+              int orderHi = qstricmp_sort(left->source()->name(),right->source()->name());
               if (orderHi!=0) return orderHi<0;
-              int orderLo = qstricmp(left->destination()->name(),right->destination()->name());
+              int orderLo = qstricmp_sort(left->destination()->name(),right->destination()->name());
               return orderLo<0;
             });
 }
@@ -795,15 +822,15 @@ FilePair *UsedDir::findFilePair(const QCString &name)
 
 DirDef *DirDefImpl::createNewDir(const QCString &path)
 {
-  ASSERT(path!=0);
+  AUTO_TRACE();
+  ASSERT(path!=nullptr);
   DirDef *dir = Doxygen::dirLinkedMap->find(path);
-  if (dir==0) // new dir
+  if (dir==nullptr) // new dir
   {
     dir = Doxygen::dirLinkedMap->add(path,
             std::unique_ptr<DirDef>(
               createDirDef(path)));
-    //printf("Adding new dir %s\n",path);
-    //printf("createNewDir %s short=%s\n",path,qPrint(dir->shortName()));
+    AUTO_TRACE_ADD("Adding new dir {} shortName {}",path,dir->shortName());
   }
   return dir;
 }
@@ -813,7 +840,7 @@ bool DirDefImpl::matchPath(const QCString &path,const StringVector &l)
   for (const auto &s : l)
   {
     std::string prefix = s.substr(0,path.length());
-    if (qstricmp(prefix.c_str(),path)==0) // case insensitive compare
+    if (qstricmp_sort(prefix.c_str(),path)==0) // case insensitive compare
     {
       return TRUE;
     }
@@ -826,21 +853,31 @@ bool DirDefImpl::matchPath(const QCString &path,const StringVector &l)
  */
 DirDef *DirDefImpl::mergeDirectoryInTree(const QCString &path)
 {
-  //printf("DirDefImpl::mergeDirectoryInTree(%s)\n",qPrint(path));
+  AUTO_TRACE("path={}",path);
   int p=0,i=0;
-  DirDef *dir=0;
+  DirDef *dir=nullptr;
   while ((i=path.find('/',p))!=-1)
   {
     QCString part=path.left(i+1);
-    if (!matchPath(part,Config_getList(STRIP_FROM_PATH)) && (part!="/" && part!="//"))
+    if (!matchPath(part,Config_getList(STRIP_FROM_PATH)) && (part!="/" && part!="//" && part!="//?/"))
     {
-      dir=createNewDir(part);
+      dir=createNewDir(removeLongPathMarker(part));
     }
     p=i+1;
   }
   return dir;
 }
 
+void DirDefImpl::overrideDirectoryGraph(bool e)
+{
+  m_hasDirectoryGraph=e;
+}
+
+bool DirDefImpl::hasDirectoryGraph() const
+{
+  return m_hasDirectoryGraph;
+}
+
 //----------------------------------------------------------------------
 
 QCString FilePair::key(const FileDef *srcFd,const FileDef *dstFd)
@@ -889,6 +926,7 @@ void DirRelation::writeDocumentation(OutputList &ol)
                       (m_src->shortName()+" &rarr; "+m_dst->dir()->shortName()));
   QCString title=theTranslator->trDirRelation(
                  (m_src->displayName()+" -> "+m_dst->dir()->shortName()));
+  AUTO_TRACE("title={}",title);
   startFile(ol,getOutputFileBase(),getOutputFileBase(),
             title,HighlightedItem::None,!generateTreeView,m_src->getOutputFileBase());
 
@@ -939,13 +977,14 @@ void DirRelation::writeDocumentation(OutputList &ol)
  */
 static void computeCommonDirPrefix()
 {
+  AUTO_TRACE();
   QCString path;
   auto it = Doxygen::dirLinkedMap->begin();
   if (!Doxygen::dirLinkedMap->empty()) // we have at least one dir
   {
     // start will full path of first dir
-    path=(*it)->name();
-    int i=path.findRev('/',path.length()-2);
+    path=removeLongPathMarker((*it)->name());
+    int i=path.findRev('/',static_cast<int>(path.length())-2);
     path=path.left(i+1);
     bool done=FALSE;
     if (i==-1)
@@ -956,16 +995,17 @@ static void computeCommonDirPrefix()
     {
       while (!done)
       {
-        uint l = path.length();
+        int l = static_cast<int>(path.length());
         size_t count=0;
         for (const auto &dir : *Doxygen::dirLinkedMap)
         {
-          QCString dirName = dir->name();
+          QCString dirName = removeLongPathMarker(dir->name());
+          //printf("dirName='%s' (l=%d) path='%s' (l=%d)\n",qPrint(dirName),dirName.length(),qPrint(path),path.length());
           if (dirName.length()>path.length())
           {
             if (dirName.left(l)!=path) // dirName does not start with path
             {
-              i=path.findRev('/',l-2);
+              i = l>=2 ? path.findRev('/',l-2) : -1;
               if (i==-1) // no unique prefix -> stop
               {
                 path="";
@@ -981,7 +1021,7 @@ static void computeCommonDirPrefix()
           else // dir is shorter than path -> take path of dir as new start
           {
             path=dir->name();
-            l=path.length();
+            l=static_cast<int>(path.length());
             i=path.findRev('/',l-2);
             if (i==-1) // no unique prefix -> stop
             {
@@ -1008,12 +1048,13 @@ static void computeCommonDirPrefix()
   {
     QCString diskName = dir->name().right(dir->name().length()-path.length());
     dir->setDiskName(diskName);
-    //printf("set disk name: %s -> %s\n",qPrint(dir->name()),qPrint(diskName));
+    AUTO_TRACE_ADD("set disk name: {} -> {}",dir->name(),diskName);
   }
 }
 
 void buildDirectories()
 {
+  AUTO_TRACE();
   // for each input file
   for (const auto &fn : *Doxygen::inputNameLinkedMap)
   {
@@ -1021,8 +1062,8 @@ void buildDirectories()
     {
       if (fd->getReference().isEmpty())
       {
-        DirDef *dir;
-        if ((dir=Doxygen::dirLinkedMap->find(fd->getPath()))==0) // new directory
+        DirDef *dir=Doxygen::dirLinkedMap->find(fd->getPath());
+        if (dir==nullptr) // new directory
         {
           dir = DirDefImpl::mergeDirectoryInTree(fd->getPath());
         }
@@ -1039,7 +1080,7 @@ void buildDirectories()
   for (const auto &dir : *Doxygen::dirLinkedMap)
   {
     QCString name = dir->name();
-    int i=name.findRev('/',name.length()-2);
+    int i=name.findRev('/',static_cast<int>(name.length())-2);
     if (i>0)
     {
       DirDef *parent = Doxygen::dirLinkedMap->find(name.left(i+1));
@@ -1047,8 +1088,8 @@ void buildDirectories()
       if (parent)
       {
         parent->addSubDir(dir.get());
-        //printf("DirDefImpl::addSubdir(): Adding subdir\n%s to\n%s\n",
-        //  qPrint(dir->displayName()), qPrint(parent->displayName()));
+        AUTO_TRACE_ADD("DirDefImpl::addSubdir(): Adding subdir {} to {}",
+                        dir->displayName(), parent->displayName());
       }
     }
   }
@@ -1060,26 +1101,26 @@ void buildDirectories()
   }
 
   // short the directories themselves
-  std::sort(Doxygen::dirLinkedMap->begin(),
+  std::stable_sort(Doxygen::dirLinkedMap->begin(),
             Doxygen::dirLinkedMap->end(),
             [](const auto &d1,const auto &d2)
             {
               QCString s1 = d1->shortName(), s2 = d2->shortName();
-              int i = qstricmp(s1,s2);
+              int i = qstricmp_sort(s1,s2);
               if (i==0) // if sort name are equal, sort on full path
               {
                 QCString n1 = d1->name(), n2 = d2->name();
-                int n = qstricmp(n1,n2);
+                int n = qstricmp_sort(n1,n2);
                 return n < 0;
               }
               return i < 0;
             });
 
-  // set the directory count identifier
-  int dirCount=0;
+  // set the directory index identifier
+  int dirIndex=0;
   for (const auto &dir : *Doxygen::dirLinkedMap)
   {
-    dir->setDirCount(dirCount++);
+    dir->setDirIndex(dirIndex++);
   }
 
   computeCommonDirPrefix();
@@ -1087,6 +1128,7 @@ void buildDirectories()
 
 void computeDirDependencies()
 {
+  AUTO_TRACE();
   // compute nesting level for each directory
   for (const auto &dir : *Doxygen::dirLinkedMap)
   {
@@ -1096,13 +1138,14 @@ void computeDirDependencies()
   // compute uses dependencies between directories
   for (const auto &dir : *Doxygen::dirLinkedMap)
   {
-    //printf("computeDependencies for %s: #dirs=%d\n",qPrint(dir->name()),Doxygen::directories.count());
+    AUTO_TRACE_ADD("computeDependencies for {}: #dirs={}",dir->name(),Doxygen::dirLinkedMap->size());
     dir->computeDependencies();
   }
 }
 
 void generateDirDocs(OutputList &ol)
 {
+  AUTO_TRACE();
   for (const auto &dir : *Doxygen::dirLinkedMap)
   {
     ol.pushGeneratorState();
@@ -1113,7 +1156,7 @@ void generateDirDocs(OutputList &ol)
     dir->writeDocumentation(ol);
     ol.popGeneratorState();
   }
-  if (Config_getBool(DIRECTORY_GRAPH))
+  //if (Config_getBool(DIRECTORY_GRAPH))
   {
     for (const auto &dr : Doxygen::dirRelations)
     {
@@ -1124,34 +1167,34 @@ void generateDirDocs(OutputList &ol)
 
 bool compareDirDefs(const DirDef *item1, const DirDef *item2)
 {
-  return qstricmp(item1->shortName(),item2->shortName()) < 0;
+  return qstricmp_sort(item1->shortName(),item2->shortName()) < 0;
 }
 
 // --- Cast functions
 
 DirDef *toDirDef(Definition *d)
 {
-  if (d==0) return 0;
+  if (d==nullptr) return nullptr;
   if (d && typeid(*d)==typeid(DirDefImpl))
   {
     return static_cast<DirDef*>(d);
   }
   else
   {
-    return 0;
+    return nullptr;
   }
 }
 
 const DirDef *toDirDef(const Definition *d)
 {
-  if (d==0) return 0;
+  if (d==nullptr) return nullptr;
   if (d && typeid(*d)==typeid(DirDefImpl))
   {
     return static_cast<const DirDef*>(d);
   }
   else
   {
-    return 0;
+    return nullptr;
   }
 }
 
