@@ -50,6 +50,9 @@
 #include "dirdef.h"
 #include "section.h"
 #include "dir.h"
+#include "growbuf.h"
+#include "outputlist.h"
+#include "moduledef.h"
 
 // no debug info
 #define Docbook_DB(x) do {} while(0)
@@ -77,11 +80,11 @@ inline void writeDocbookString(TextStream &t,const QCString &s)
   t << convertToDocBook(s);
 }
 
-inline void writeDocbookCodeString(TextStream &t,const QCString &str, int &col)
+inline void writeDocbookCodeString(TextStream &t,const QCString &str, size_t &col)
 {
   if (str.isEmpty()) return;
   const char *s = str.data();
-  char c;
+  char c=0;
   while ((c=*s++))
   {
     switch(c)
@@ -102,7 +105,7 @@ inline void writeDocbookCodeString(TextStream &t,const QCString &str, int &col)
       case '"':  t << "&quot;"; col++; break;
       default:
         {
-          uchar uc = static_cast<uchar>(c);
+          uint8_t uc = static_cast<uint8_t>(c);
           static const char *hex="0123456789ABCDEF";
           if (uc<32)
           {
@@ -143,14 +146,14 @@ void writeDocbookLink(TextStream &t,const QCString & /*extRef*/,const QCString &
   t << "</link>";
 }
 
-DocbookCodeGenerator::DocbookCodeGenerator(TextStream &t) : m_t(t)
+DocbookCodeGenerator::DocbookCodeGenerator(TextStream *t) : m_t(t)
 {
 }
 
 void DocbookCodeGenerator::codify(const QCString &text)
 {
   Docbook_DB(("(codify \"%s\")\n",text));
-  writeDocbookCodeString(m_t,text,m_col);
+  writeDocbookCodeString(*m_t,text,m_col);
 }
 
 void DocbookCodeGenerator::writeCodeLink(CodeSymbolType,
@@ -159,7 +162,7 @@ void DocbookCodeGenerator::writeCodeLink(CodeSymbolType,
     const QCString &tooltip)
 {
   Docbook_DB(("(writeCodeLink)\n"));
-  writeDocbookLink(m_t,ref,file,anchor,name,tooltip);
+  writeDocbookLink(*m_t,ref,file,anchor,name,tooltip);
   m_col+=name.length();
 }
 
@@ -170,10 +173,10 @@ void DocbookCodeGenerator::writeCodeLinkLine(CodeSymbolType,
 {
   Docbook_DB(("(writeCodeLinkLine)\n"));
   if (!writeLineAnchor) return;
-  m_t << "<anchor xml:id=\"_" << stripExtensionGeneral(stripPath(file),".xml");
-  m_t << "_1l";
-  writeDocbookString(m_t,name);
-  m_t << "\"/>";
+  *m_t << "<anchor xml:id=\"_" << stripExtensionGeneral(stripPath(file),".xml");
+  *m_t << "_1l";
+  writeDocbookString(*m_t,name);
+  *m_t << "\"/>";
   m_col+=name.length();
 }
 
@@ -184,7 +187,7 @@ void DocbookCodeGenerator::writeTooltip(const QCString &, const DocLinkInfo &, c
   Docbook_DB(("(writeToolTip)\n"));
 }
 
-void DocbookCodeGenerator::startCodeLine(bool)
+void DocbookCodeGenerator::startCodeLine(int)
 {
   Docbook_DB(("(startCodeLine)\n"));
   m_insideCodeLine=TRUE;
@@ -193,25 +196,25 @@ void DocbookCodeGenerator::startCodeLine(bool)
 
 void DocbookCodeGenerator::endCodeLine()
 {
-  if (m_insideCodeLine) m_t << "\n";
+  if (m_insideCodeLine) *m_t << "\n";
   Docbook_DB(("(endCodeLine)\n"));
   m_lineNumber = -1;
-  m_refId.resize(0);
-  m_external.resize(0);
+  m_refId.clear();
+  m_external.clear();
   m_insideCodeLine=FALSE;
 }
 
 void DocbookCodeGenerator::startFontClass(const QCString &colorClass)
 {
   Docbook_DB(("(startFontClass)\n"));
-  m_t << "<emphasis role=\"" << colorClass << "\">";
+  *m_t << "<emphasis role=\"" << colorClass << "\">";
   m_insideSpecialHL=TRUE;
 }
 
 void DocbookCodeGenerator::endFontClass()
 {
   Docbook_DB(("(endFontClass)\n"));
-  m_t << "</emphasis>"; // non DocBook
+  *m_t << "</emphasis>"; // non DocBook
   m_insideSpecialHL=FALSE;
 }
 
@@ -242,11 +245,11 @@ void DocbookCodeGenerator::writeLineNumber(const QCString &ref,const QCString &f
     {
       codify(lineNumber);
     }
-    m_t << " ";
+    *m_t << " ";
   }
   else
   {
-    m_t << l << " ";
+    *m_t << l << " ";
   }
   m_col=0;
 }
@@ -258,48 +261,73 @@ void DocbookCodeGenerator::finish()
 
 void DocbookCodeGenerator::startCodeFragment(const QCString &)
 {
-DB_GEN_C1(m_t)
-  m_t << "<programlisting linenumbering=\"unnumbered\">";
+DB_GEN_C1(*m_t)
+  *m_t << "<programlisting linenumbering=\"unnumbered\">";
 }
 
 void DocbookCodeGenerator::endCodeFragment(const QCString &)
 {
-DB_GEN_C1(m_t)
+DB_GEN_C1(*m_t)
   //endCodeLine checks is there is still an open code line, if so closes it.
   endCodeLine();
 
-  m_t << "</programlisting>";
+  *m_t << "</programlisting>";
 }
 
 //-------------------------------------------------------------------------------
 
 DocbookGenerator::DocbookGenerator()
   : OutputGenerator(Config_getString(DOCBOOK_OUTPUT))
-  , m_codeGen(m_t)
+  , m_codeList(std::make_unique<OutputCodeList>())
 {
 DB_GEN_C
+  m_codeGen = m_codeList->add<DocbookCodeGenerator>(&m_t);
 }
 
-DocbookGenerator::DocbookGenerator(const DocbookGenerator &og)
-  : OutputGenerator(og)
-  , m_codeGen(m_t)
+DocbookGenerator::DocbookGenerator(const DocbookGenerator &og) : OutputGenerator(og.m_dir), OutputGenIntf()
 {
+  m_codeList         = std::make_unique<OutputCodeList>(*og.m_codeList);
+  m_codeGen          = m_codeList->get<DocbookCodeGenerator>(OutputType::Docbook);
+  m_codeGen->setTextStream(&m_t);
+  m_denseText        = og.m_denseText;
+  m_inGroup          = og.m_inGroup;
+  m_levelListItem    = og.m_levelListItem;
+  m_inListItem       = og.m_inListItem;
+  m_inSimpleSect     = og.m_inSimpleSect;
+  m_descTable        = og.m_descTable;
+  m_simpleTable      = og.m_simpleTable;
+  m_inLevel          = og.m_inLevel;
+  m_firstMember      = og.m_firstMember;
+  m_openSectionCount = og.m_openSectionCount;
 }
 
 DocbookGenerator &DocbookGenerator::operator=(const DocbookGenerator &og)
 {
-  OutputGenerator::operator=(og);
+  if (this!=&og)
+  {
+    m_dir              = og.m_dir;
+    m_codeList         = std::make_unique<OutputCodeList>(*og.m_codeList);
+    m_codeGen          = m_codeList->get<DocbookCodeGenerator>(OutputType::Docbook);
+    m_codeGen->setTextStream(&m_t);
+    m_denseText        = og.m_denseText;
+    m_inGroup          = og.m_inGroup;
+    m_levelListItem    = og.m_levelListItem;
+    m_inListItem       = og.m_inListItem;
+    m_inSimpleSect     = og.m_inSimpleSect;
+    m_descTable        = og.m_descTable;
+    m_simpleTable      = og.m_simpleTable;
+    m_inLevel          = og.m_inLevel;
+    m_firstMember      = og.m_firstMember;
+    m_openSectionCount = og.m_openSectionCount;
+  }
   return *this;
 }
 
-std::unique_ptr<OutputGenerator> DocbookGenerator::clone() const
-{
-  return std::make_unique<DocbookGenerator>(*this);
-}
+DocbookGenerator::~DocbookGenerator() = default;
 
-DocbookGenerator::~DocbookGenerator()
+void DocbookGenerator::addCodeGen(OutputCodeList &list)
 {
-DB_GEN_C
+  list.add<DocbookCodeGeneratorDefer>(m_codeGen);
 }
 
 void DocbookGenerator::init()
@@ -321,7 +349,7 @@ void DocbookGenerator::cleanup()
 }
 
 
-void DocbookGenerator::startFile(const QCString &name,const QCString &,const QCString &,int)
+void DocbookGenerator::startFile(const QCString &name,const QCString &,const QCString &,int,int)
 {
 DB_GEN_C
   QCString fileName=name;
@@ -341,10 +369,11 @@ DB_GEN_C
   relPath = relativePathToRoot(fileName);
   if (!fileName.endsWith(".xml")) fileName+=".xml";
   startPlainFile(fileName);
-  m_codeGen.setRelativePath(relPath);
-  m_codeGen.setSourceFileName(stripPath(fileName));
+  m_codeGen->setRelativePath(relPath);
+  m_codeGen->setSourceFileName(stripPath(fileName));
+  m_pageLinks = QCString();
 
-  m_t << "<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n";;
+  m_t << "<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n";
   m_t << "<" << fileType << " xmlns=\"http://docbook.org/ns/docbook\" version=\"5.0\" xmlns:xlink=\"http://www.w3.org/1999/xlink\"";
   if (!pageName.isEmpty()) m_t << " xml:id=\"_" <<  stripPath(pageName) << "\"";
   m_t << " xml:lang=\"" << theTranslator->trISOLang() << "\"";
@@ -358,8 +387,11 @@ DB_GEN_C
   m_inLevel = -1;
   m_inGroup = FALSE;
 
+  // Write page links only after all sections have been closed to avoid bugs
+  m_t << m_pageLinks;
+
   QCString fileType="section";
-  QCString fileName= m_codeGen.sourceFileName();
+  QCString fileName= m_codeGen->sourceFileName();
   if (fileName == "index.xml")
   {
     fileType="book";
@@ -370,7 +402,7 @@ DB_GEN_C
   }
   m_t << "</" << fileType << ">\n";
   endPlainFile();
-  m_codeGen.setSourceFileName("");
+  m_codeGen->setSourceFileName("");
 }
 
 void DocbookGenerator::startIndexSection(IndexSection is)
@@ -389,12 +421,13 @@ DB_GEN_C2("IndexSection " << is)
     case IndexSection::isTitlePageAuthor:
       break;
     case IndexSection::isMainPage:
-      m_t << "<chapter>\n";
-      m_t << "    <title>";
       break;
     case IndexSection::isModuleIndex:
       //Module Index\n"
       break;
+    case IndexSection::isTopicIndex:
+      //Module Index\n"
+      break;
     case IndexSection::isDirIndex:
       //Directory Index\n"
       break;
@@ -420,6 +453,10 @@ DB_GEN_C2("IndexSection " << is)
       m_t << "<chapter>\n";
       m_t << "    <title>";
       break;
+    case IndexSection::isTopicDocumentation:
+      m_t << "<chapter>\n";
+      m_t << "    <title>";
+      break;
     case IndexSection::isDirDocumentation:
       m_t << "<chapter>\n";
       m_t << "    <title>";
@@ -463,13 +500,19 @@ DB_GEN_C2("IndexSection " << is)
     case IndexSection::isTitlePageAuthor:
       break;
     case IndexSection::isMainPage:
-      m_t << "</title>\n";
-      m_t << "    <xi:include href=\"mainpage.xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
-      m_t << "</chapter>\n";
+      {
+        if (Doxygen::mainPage)
+        {
+          writePageLink(QCString("mainpage"), TRUE);
+        }
+      }
       break;
     case IndexSection::isModuleIndex:
       //m_t << "</chapter>\n";
       break;
+    case IndexSection::isTopicIndex:
+      //m_t << "</chapter>\n";
+      break;
     case IndexSection::isDirIndex:
       //m_t << "<xi:include href=\"dirs.xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>";
       //m_t << "</chapter>\n";
@@ -497,19 +540,33 @@ DB_GEN_C2("IndexSection " << is)
       //m_t << "<xi:include href=\"pages.xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>";
       //m_t << "</chapter>\n";
       break;
-    case IndexSection::isModuleDocumentation:
+    case IndexSection::isTopicDocumentation:
       {
         m_t << "</title>\n";
         for (const auto &gd : *Doxygen::groupLinkedMap)
         {
-          if (!gd->isReference())
+          if (!gd->isReference() && !gd->isASubGroup())
           {
-            m_t << "    <xi:include href=\"" << gd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
+            writePageLink(gd->getOutputFileBase(), TRUE);
           }
         }
       }
       m_t << "</chapter>\n";
       break;
+    case IndexSection::isModuleDocumentation:
+      {
+        m_t << "</title>\n";
+        for (const auto &mod : ModuleManager::instance().modules())
+        {
+          if (!mod->isReference() && mod->isPrimaryInterface())
+          {
+            writePageLink(mod->getOutputFileBase(), TRUE);
+          }
+        }
+      }
+      m_t << "</chapter>\n";
+      break;
+      break;
     case IndexSection::isDirDocumentation:
       {
         m_t << "</title>\n";
@@ -555,7 +612,7 @@ DB_GEN_C2("IndexSection " << is)
         for (const auto &cd : *Doxygen::classLinkedMap)
         {
           if (cd->isLinkableInProject() &&
-              cd->templateMaster()==0 &&
+              cd->templateMaster()==nullptr &&
              !cd->isEmbeddedInOuterScope() &&
              !cd->isAlias()
              )
@@ -597,6 +654,14 @@ DB_GEN_C2("IndexSection " << is)
       m_t << "</chapter>\n";
       break;
     case IndexSection::isPageDocumentation:
+        for (const auto &pd : *Doxygen::pageLinkedMap)
+        {
+          if (!pd->getGroupDef() && !pd->isReference() && !pd->hasParentPage()
+            && Doxygen::mainPage.get() != pd.get())
+          {
+            writePageLink(pd->getOutputFileBase(), TRUE);
+          }
+        }
       break;
     case IndexSection::isPageDocumentation2:
       break;
@@ -605,26 +670,17 @@ DB_GEN_C2("IndexSection " << is)
       break;
   }
 }
-void DocbookGenerator::writePageLink(const QCString &name, bool /*first*/)
+void DocbookGenerator::writePageLink(const QCString &name, bool first)
 {
 DB_GEN_C
-  for (const auto &pd : *Doxygen::pageLinkedMap)
-  {
-    if (!pd->getGroupDef() && !pd->isReference() && pd->name() == stripPath(name))
-    {
-      m_t << "<chapter>\n";
-      if (pd->hasTitle())
-      {
-        m_t << "    <title>" << convertToDocBook(pd->title()) << "</title>\n";
-      }
-      else
-      {
-        m_t << "    <title>" << convertToDocBook(pd->name()) << "</title>\n";
-      }
-      m_t << "    <xi:include href=\"" << pd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
-      m_t << "</chapter>\n";
-    }
-  }
+  QCString link;
+  link.sprintf("    <xi:include href=\"%s.xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n",
+               name.data());
+  if (first)
+    m_t << link;
+  else
+    // Buffer page links and write them after all sections are closed
+    m_pageLinks += link;
 }
 
 void DocbookGenerator::writeDoc(const IDocNodeAST *ast,const Definition *ctx,const MemberDef *,int)
@@ -633,7 +689,7 @@ DB_GEN_C
   auto astImpl = dynamic_cast<const DocNodeAST*>(ast);
   if (astImpl)
   {
-    DocbookDocVisitor visitor(m_t,m_codeGen,ctx?ctx->getDefFileExtension():QCString());
+    DocbookDocVisitor visitor(m_t,*m_codeList,ctx?ctx->getDefFileExtension():QCString());
     std::visit(visitor,astImpl->root);
   }
 }
@@ -712,14 +768,14 @@ DB_GEN_C
   if (m_inSimpleSect[m_levelListItem]) m_t << "</simplesect>\n";
   m_inSimpleSect[m_levelListItem] = FALSE;
 }
-void DocbookGenerator::startMemberItem(const QCString &,int,const QCString &)
+void DocbookGenerator::startMemberItem(const QCString &,MemberItemType,const QCString &)
 {
 DB_GEN_C
   if (m_inListItem[m_levelListItem]) m_t << "</listitem>\n";
   m_t << "            <listitem><para>";
   m_inListItem[m_levelListItem] = TRUE;
 }
-void DocbookGenerator::endMemberItem()
+void DocbookGenerator::endMemberItem(MemberItemType)
 {
 DB_GEN_C
   m_t << "</para>\n";
@@ -860,6 +916,10 @@ void DocbookGenerator::endDoxyAnchor(const QCString &,const QCString &)
 {
 DB_GEN_C
 }
+void DocbookGenerator::addLabel(const QCString &,const QCString &)
+{
+DB_GEN_C
+}
 void DocbookGenerator::startMemberDocName(bool)
 {
 DB_GEN_C
@@ -937,12 +997,12 @@ void DocbookGenerator::endExamples()
 DB_GEN_C
   m_t << "</simplesect>\n";
 }
-void DocbookGenerator::startSubsubsection()
+void DocbookGenerator::startCompoundTemplateParams()
 {
 DB_GEN_C
   m_t << "<simplesect><title>";
 }
-void DocbookGenerator::endSubsubsection()
+void DocbookGenerator::endCompoundTemplateParams()
 {
 DB_GEN_C
   m_t << "</title></simplesect>\n";
@@ -978,19 +1038,46 @@ DB_GEN_C
   }
   m_t << " ";
 }
+
 void DocbookGenerator::startParameterName(bool)
 {
 DB_GEN_C
   m_t << " ";
 }
-void DocbookGenerator::endParameterName(bool last,bool /*emptyList*/,bool closeBracket)
+
+void DocbookGenerator::endParameterName()
 {
 DB_GEN_C
-  if (last)
+}
+
+void DocbookGenerator::startParameterExtra()
+{
+DB_GEN_C
+}
+
+void DocbookGenerator::endParameterExtra(bool last,bool /*emptyList*/,bool closeBracket)
+{
+DB_GEN_C
+  if (last && closeBracket)
   {
-    if (closeBracket) m_t << ")";
+    m_t << ")";
   }
 }
+
+
+void DocbookGenerator::startParameterDefVal(const char *sep)
+{
+DB_GEN_C
+  m_t << sep;
+  if (!m_denseText) m_t << "<computeroutput>";
+}
+
+void DocbookGenerator::endParameterDefVal()
+{
+DB_GEN_C
+  if (!m_denseText) m_t << "</computeroutput>\n";
+}
+
 void DocbookGenerator::startMemberTemplateParams()
 {
 DB_GEN_C
@@ -1023,7 +1110,7 @@ DB_GEN_C
 void DocbookGenerator::startMemberDocSimple(bool isEnum)
 {
 DB_GEN_C
-  int ncols;
+  int ncols=0;
   QCString title;
   if (isEnum)
   {
@@ -1046,7 +1133,7 @@ DB_GEN_C
   m_simpleTable = true;
 }
 
-void DocbookGenerator::endMemberDocSimple(bool isEnum)
+void DocbookGenerator::endMemberDocSimple(bool /* isEnum */)
 {
 DB_GEN_C
   m_t << "    </tbody>\n";
@@ -1266,10 +1353,133 @@ void DocbookGenerator::closeAllSections()
 }
 
 void DocbookGenerator::writeInheritedSectionTitle(
-                  const QCString &id,    const QCString &ref,
+                  const QCString &/*id*/,const QCString &ref,
                   const QCString &file,  const QCString &anchor,
                   const QCString &title, const QCString &name)
 {
 DB_GEN_C
   m_t << theTranslator->trInheritedFrom(convertToDocBook(title), objectLinkToString(ref, file, anchor, name));
 }
+
+void DocbookGenerator::writeLocalToc(const SectionRefs &sectionRefs,const LocalToc &localToc)
+{
+  if (localToc.isDocbookEnabled())
+  {
+    m_t << "    <toc>\n";
+    m_t << "    <title>" << theTranslator->trRTFTableOfContents() << "</title>\n";
+    int level=1;
+    int maxLevel = localToc.docbookLevel();
+    BoolVector inLi(maxLevel+1,false);
+    for (const SectionInfo *si : sectionRefs)
+    {
+      SectionType type = si->type();
+      if (type.isSection())
+      {
+        //printf("  level=%d title=%s\n",level,qPrint(si->title));
+        int nextLevel = type.level();
+        if (nextLevel>level)
+        {
+          for (int l=level;l<nextLevel;l++)
+          {
+            if (l < maxLevel) m_t << "    <tocdiv>\n";
+          }
+        }
+        else if (nextLevel<level)
+        {
+          for (int l=level;l>nextLevel;l--)
+          {
+            inLi[l]=FALSE;
+            if (l <= maxLevel) m_t << "    </tocdiv>\n";
+          }
+        }
+        if (nextLevel <= maxLevel)
+        {
+          QCString titleDoc = convertToDocBook(si->title());
+          QCString label    = convertToDocBook(si->label());
+          if (titleDoc.isEmpty()) titleDoc = label;
+          m_t << "      <tocentry>" << titleDoc << "</tocentry>\n";
+        }
+        inLi[nextLevel]=TRUE;
+        level = nextLevel;
+      }
+    }
+    if (level > maxLevel) level = maxLevel;
+    while (level>1 && level <= maxLevel)
+    {
+      inLi[level]=FALSE;
+      m_t << "</tocdiv>\n";
+      level--;
+    }
+    m_t << "    </toc>\n";
+  }
+}
+
+//-------------------------------------------------------------------------------------------------
+
+static constexpr auto hex="0123456789ABCDEF";
+
+/*! Converts a string to an DocBook-encoded string */
+QCString convertToDocBook(const QCString &s, const bool retainNewline)
+{
+  if (s.isEmpty()) return s;
+  GrowBuf growBuf;
+  const char *p = s.data();
+  const char *q = nullptr;
+  int cnt = 0;
+  char c = 0;
+  while ((c=*p++))
+  {
+    switch (c)
+    {
+      case '\n': if (retainNewline) growBuf.addStr("<literallayout>&#160;&#xa;</literallayout>"); growBuf.addChar(c);   break;
+      case '<':  growBuf.addStr("&lt;");   break;
+      case '>':  growBuf.addStr("&gt;");   break;
+      case '&':  // possibility to have a special symbol
+        q = p;
+        cnt = 2; // we have to count & and ; as well
+        while ((*q >= 'a' && *q <= 'z') || (*q >= 'A' && *q <= 'Z') || (*q >= '0' && *q <= '9'))
+        {
+          cnt++;
+          q++;
+        }
+        if (*q == ';')
+        {
+           --p; // we need & as well
+           HtmlEntityMapper::SymType res = HtmlEntityMapper::instance().name2sym(QCString(p).left(cnt));
+           if (res == HtmlEntityMapper::Sym_Unknown)
+           {
+             p++;
+             growBuf.addStr("&amp;");
+           }
+           else
+           {
+             growBuf.addStr(HtmlEntityMapper::instance().docbook(res));
+             q++;
+             p = q;
+           }
+        }
+        else
+        {
+          growBuf.addStr("&amp;");
+        }
+        break;
+      case '\'': growBuf.addStr("&apos;"); break;
+      case '"':  growBuf.addStr("&quot;"); break;
+      case  1: case  2: case  3: case  4: case  5: case  6: case 7:  case  8:
+      case 11: case 12: case 14: case 15: case 16: case 17: case 18:
+      case 19: case 20: case 21: case 22: case 23: case 24: case 25: case 26:
+      case 27: case 28: case 29: case 30: case 31:
+        growBuf.addStr("&#x24");
+        growBuf.addChar(hex[static_cast<uint8_t>(c)>>4]);
+        growBuf.addChar(hex[static_cast<uint8_t>(c)&0xF]);
+        growBuf.addChar(';');
+        break;
+      default:
+        growBuf.addChar(c);
+        break;
+    }
+  }
+  growBuf.addChar(0);
+  return growBuf.get();
+}
+
