@@ -25,18 +25,16 @@
 #include "define.h"
 #include "cache.h"
 #include "symbolmap.h"
+#include "searchindex.h"
 
 #define THREAD_LOCAL thread_local
 #define AtomicInt    std::atomic_int
-#define AtomicBool   std::atomic_bool
 
 class RefList;
 class PageLinkedMap;
 class PageDef;
-class SearchIndexIntf;
 class ParserManager;
 class BufStr;
-class CiteDict;
 class MemberDef;
 class GroupDef;
 class GroupLinkedMap;
@@ -50,9 +48,6 @@ class NamespaceLinkedMap;
 class NamespaceDef;
 class DirRelationLinkedMap;
 class IndexList;
-class FormulaList;
-class FormulaDict;
-class FormulaNameDict;
 class Preprocessor;
 struct MemberGroupInfo;
 class NamespaceDefMutable;
@@ -60,10 +55,10 @@ class NamespaceDefMutable;
 struct LookupInfo
 {
   LookupInfo() = default;
-  LookupInfo(const Definition *d,const MemberDef *td,QCString ts,QCString rt)
-    : definition(d), typeDef(td), templSpec(ts),resolvedType(rt) {}
-  const Definition  *definition = 0;
-  const MemberDef *typeDef = 0;
+  LookupInfo(const Definition *d,const MemberDef *td,const QCString &ts,const QCString &rt)
+    : definition(d), typeDef(td), templSpec(ts), resolvedType(rt) {}
+  const Definition  *definition = nullptr;
+  const MemberDef *typeDef = nullptr;
   QCString   templSpec;
   QCString   resolvedType;
 };
@@ -80,6 +75,8 @@ using InputFileEncodingList = std::vector<InputFileEncoding>;
 
 using ClangUsrMap = std::unordered_map<std::string,const Definition *>;
 
+using StaticInitMap = std::unordered_map<std::string,BodyInfo>;
+
 /*! \brief This class serves as a namespace for global variables used by doxygen.
  *
  *  All fields in this class are public and static, so they can be used directly.
@@ -93,10 +90,9 @@ class Doxygen
     static PageLinkedMap            *exampleLinkedMap;
     static PageLinkedMap            *pageLinkedMap;
     static std::unique_ptr<PageDef>  mainPage;
-    static bool                      insideMainPage;
     static FileNameLinkedMap        *includeNameLinkedMap;
     static FileNameLinkedMap        *exampleNameLinkedMap;
-    static StringSet                 inputPaths;
+    static StringUnorderedSet        inputPaths;
     static FileNameLinkedMap        *inputNameLinkedMap;
     static FileNameLinkedMap        *imageNameLinkedMap;
     static FileNameLinkedMap        *dotFileNameLinkedMap;
@@ -108,13 +104,14 @@ class Doxygen
     static GroupLinkedMap           *groupLinkedMap;
     static NamespaceLinkedMap       *namespaceLinkedMap;
     static StringMap                 tagDestinationMap;
-    static StringMap                 aliasMap;
+    static StringUnorderedSet        tagFileSet;
     static MemberGroupInfoMap        memberGroupInfoMap;
     static StringUnorderedSet        expandAsDefinedSet;
+    static std::unique_ptr<NamespaceDef> globalNamespaceDef;
     static NamespaceDefMutable      *globalScope;
     static QCString                  htmlFileExtension;
     static bool                      parseSourcesNeeded;
-    static SearchIndexIntf          *searchIndex;
+    static SearchIndexIntf           searchIndex;
     static SymbolMap<Definition>    *symbolMap;
     static ClangUsrMap              *clangUsrMap;
     static Cache<std::string,LookupInfo> *typeLookupCache;
@@ -125,7 +122,6 @@ class Doxygen
     static bool                      suppressDocWarnings;
     static QCString                  filterDBFileName;
     static IndexList                *indexList;
-    static int                       subpageNestingLevel;
     static QCString                  spaces;
     static bool                      generatingXmlOutput;
     static DefinesPerFileList        macroDefinitions;
@@ -133,29 +129,17 @@ class Doxygen
     static QCString                  verifiedDotPath;
     static volatile bool             terminating;
     static InputFileEncodingList     inputFileEncodingList;
-    static std::mutex                searchIndexMutex;
     static std::mutex                countFlowKeywordsMutex;
     static std::mutex                addExampleMutex;
+    static StaticInitMap             staticInitMap;
 };
 
-/** Deleter that only deletes an object if doxygen is not already terminating */
-template<class T>
-struct NonTerminatingDeleter
-{
-  void operator()(T *obj)
-  {
-    if (!Doxygen::terminating) delete obj;
-  }
-};
-
-
 void initDoxygen();
 void readConfiguration(int argc, char **argv);
 void checkConfiguration();
 void adjustConfiguration();
 void parseInput();
 void generateOutput();
-void readAliases();
 void cleanUpDoxygen();
 void readFileOrDirectory(const QCString &s,
                         FileNameLinkedMap *fnDict,
@@ -166,9 +150,8 @@ void readFileOrDirectory(const QCString &s,
                         StringUnorderedSet *resultSet,
                         bool recursive,
                         bool errorIfNotExist=TRUE,
-                        StringUnorderedSet *killSet = 0,
-                        StringSet *paths = 0
+                        StringUnorderedSet *killSet = nullptr,
+                        StringUnorderedSet *paths = nullptr
                        );
-void copyAndFilterFile(const char *fileName,BufStr &dest);
 
 #endif
