@@ -18,6 +18,7 @@
 #include "language.h"
 #include "doxygen.h"
 #include "outputgen.h"
+#include "outputlist.h"
 #include "dot.h"
 #include "message.h"
 #include "config.h"
@@ -37,10 +38,11 @@
 #include "indexlist.h"
 #include "growbuf.h"
 #include "portable.h"
+#include "codefragment.h"
 
 static const int NUM_HTML_LIST_TYPES = 4;
 static const char types[][NUM_HTML_LIST_TYPES] = {"1", "a", "i", "A"};
-enum contexts_t
+enum class contexts_t
 {
     NONE,      // 0
     STARTLI,   // 1
@@ -53,18 +55,24 @@ enum contexts_t
     INTERDD,   // 8
     INTERTD    // 9
 };
-static const char *contexts[10] =
-{ "",          // 0
-  "startli",   // 1
-  "startdd",   // 2
-  "endli",     // 3
-  "enddd",     // 4
-  "starttd",   // 5
-  "endtd",     // 6
-  "interli",   // 7
-  "interdd",   // 8
-  "intertd"    // 9
-};
+
+static constexpr const char *contexts(contexts_t type)
+{
+  switch (type)
+  {
+    case contexts_t::NONE:    return nullptr;
+    case contexts_t::STARTLI: return "startli";
+    case contexts_t::STARTDD: return "startdd";
+    case contexts_t::ENDLI:   return "endli";
+    case contexts_t::ENDDD:   return "enddd";
+    case contexts_t::STARTTD: return "starttd";
+    case contexts_t::ENDTD:   return "endtd";
+    case contexts_t::INTERLI: return "interli";
+    case contexts_t::INTERDD: return "interdd";
+    case contexts_t::INTERTD: return "intertd";
+    default:                  return nullptr;
+  }
+}
 static const char *hex="0123456789ABCDEF";
 
 static QCString convertIndexWordToAnchor(const QCString &word)
@@ -76,7 +84,7 @@ static QCString convertIndexWordToAnchor(const QCString &word)
   result += "_";
   cnt++;
   const char *str = word.data();
-  unsigned char c;
+  unsigned char c = 0;
   if (str)
   {
     while ((c = *str++))
@@ -94,7 +102,7 @@ static QCString convertIndexWordToAnchor(const QCString &word)
       else
       {
         char enc[4];
-        enc[0] = ':';
+        enc[0] = ';';
         enc[1] = hex[(c & 0xf0) >> 4];
         enc[2] = hex[c & 0xf];
         enc[3] = 0;
@@ -119,6 +127,9 @@ static bool mustBeOutsideParagraph(const DocNodeVariant &n)
                                  /* <hr> */         DocHorRuler,
                                  /* <blockquote> */ DocHtmlBlockQuote,
                                  /* \parblock */    DocParBlock,
+                                 /* \dotfile */     DocDotFile,
+                                 /* \mscfile */     DocMscFile,
+                                 /* \diafile */     DocDiaFile,
                                  /* <details> */    DocHtmlDetails,
                                  /* <summary> */    DocHtmlSummary,
                                                     DocIncOperator >(n))
@@ -232,7 +243,7 @@ static void mergeHtmlAttributes(const HtmlAttribList &attribs, HtmlAttribList &m
   }
 }
 
-static QCString htmlAttribsToString(const HtmlAttribList &attribs, QCString *pAltValue = 0)
+static QCString htmlAttribsToString(const HtmlAttribList &attribs, QCString *pAltValue = nullptr)
 {
   QCString result;
   for (const auto &att : attribs)
@@ -273,11 +284,34 @@ static QCString htmlAttribsToString(const HtmlAttribList &attribs, QCString *pAl
   return result;
 }
 
+static QCString makeShortName(const QCString &name)
+{
+  QCString shortName = name;
+  int i = shortName.findRev('/');
+  if (i!=-1)
+  {
+    shortName=shortName.mid(i+1);
+  }
+  return shortName;
+}
+
+static QCString makeBaseName(const QCString &name)
+{
+  QCString baseName = makeShortName(name);
+  int i=baseName.find('.');
+  if (i!=-1)
+  {
+    baseName=baseName.left(i);
+  }
+  return baseName;
+}
+
+
 //-------------------------------------------------------------------------
 
-HtmlDocVisitor::HtmlDocVisitor(TextStream &t,CodeOutputInterface &ci,
-                               const Definition *ctx)
-  : m_t(t), m_ci(ci), m_ctx(ctx)
+HtmlDocVisitor::HtmlDocVisitor(TextStream &t,OutputCodeList &ci,
+                               const Definition *ctx,const QCString &fn)
+  : m_t(t), m_ci(ci), m_ctx(ctx), m_fileName(fn)
 {
   if (ctx) m_langExt=ctx->getDefFileExtension();
 }
@@ -377,11 +411,11 @@ void HtmlDocVisitor::writeObfuscatedMailAddress(const QCString &url)
     if (!url.isEmpty())
     {
       const char *p = url.data();
-      uint size=3;
+      uint32_t size=3;
       while (*p)
       {
         m_t << "+'";
-        for (uint j=0;j<size && *p;j++)
+        for (uint32_t j=0;j<size && *p;j++)
         {
           p = writeUTF8Char(m_t,p);
         }
@@ -409,10 +443,10 @@ void HtmlDocVisitor::operator()(const DocURL &u)
     {
       const char *p = url.data();
       // also obfuscate the address as shown on the web page
-      uint size=5;
+      uint32_t size=5;
       while (*p)
       {
-        for (uint j=0;j<size && *p;j++)
+        for (uint32_t j=0;j<size && *p;j++)
         {
           p = writeUTF8Char(m_t,p);
         }
@@ -552,11 +586,11 @@ void HtmlDocVisitor::operator()(const DocVerbatim &s)
                                         langExt,
                                         s.isExample(),
                                         s.exampleFile(),
-                                        0,     // fileDef
+                                        nullptr,     // fileDef
                                         -1,    // startLine
                                         -1,    // endLine
                                         FALSE, // inlineFragment
-                                        0,     // memberDef
+                                        nullptr,     // memberDef
                                         TRUE,  // show line numbers
                                         m_ctx  // search context
                                        );
@@ -596,7 +630,7 @@ void HtmlDocVisitor::operator()(const DocVerbatim &s)
     case DocVerbatim::Dot:
       {
         static int dotindex = 1;
-        QCString fileName(4096);
+        QCString fileName(4096, QCString::ExplicitSize);
 
         forceEndParagraph(s);
         fileName.sprintf("%s%d%s",
@@ -630,7 +664,7 @@ void HtmlDocVisitor::operator()(const DocVerbatim &s)
         forceEndParagraph(s);
 
         static int mscindex = 1;
-        QCString baseName(4096);
+        QCString baseName(4096, QCString::ExplicitSize);
 
         baseName.sprintf("%s%d",
             qPrint(Config_getString(HTML_OUTPUT)+"/inline_mscgraph_"),
@@ -704,11 +738,11 @@ void HtmlDocVisitor::operator()(const DocInclude &inc)
                                         langExt,
                                         inc.isExample(),
                                         inc.exampleFile(),
-                                        0,     // fileDef
+                                        nullptr,     // fileDef
                                         -1,    // startLine
                                         -1,    // endLine
                                         TRUE,  // inlineFragment
-                                        0,     // memberDef
+                                        nullptr,     // memberDef
                                         FALSE, // show line numbers
                                         m_ctx  // search context
                                        );
@@ -720,22 +754,21 @@ void HtmlDocVisitor::operator()(const DocInclude &inc)
          forceEndParagraph(inc);
          m_ci.startCodeFragment("DoxyCode");
          FileInfo cfi( inc.file().str() );
-         FileDef *fd = createFileDef( cfi.dirPath(), cfi.fileName() );
+         auto fd = createFileDef( cfi.dirPath(), cfi.fileName() );
          getCodeParser(inc.extension()).parseCode(m_ci,
                                            inc.context(),
                                            inc.text(),
                                            langExt,
                                            inc.isExample(),
                                            inc.exampleFile(),
-                                           fd,   // fileDef,
+                                           fd.get(),   // fileDef,
                                            -1,    // start line
                                            -1,    // end line
                                            FALSE, // inline fragment
-                                           0,     // memberDef
+                                           nullptr,     // memberDef
                                            TRUE,  // show line numbers
                                            m_ctx  // search context
                                            );
-         delete fd;
          m_ci.endCodeFragment("DoxyCode");
          forceStartParagraph(inc);
       }
@@ -763,56 +796,19 @@ void HtmlDocVisitor::operator()(const DocInclude &inc)
       forceStartParagraph(inc);
       break;
     case DocInclude::Snippet:
-      {
-         forceEndParagraph(inc);
-         m_ci.startCodeFragment("DoxyCode");
-         getCodeParser(inc.extension()).parseCode(m_ci,
-                                           inc.context(),
-                                           extractBlock(inc.text(),inc.blockId()),
-                                           langExt,
-                                           inc.isExample(),
-                                           inc.exampleFile(),
-                                           0,
-                                           -1,    // startLine
-                                           -1,    // endLine
-                                           TRUE,  // inlineFragment
-                                           0,     // memberDef
-                                           FALSE, // show line number
-                                           m_ctx  // search context
-                                          );
-         m_ci.endCodeFragment("DoxyCode");
-         forceStartParagraph(inc);
-      }
-      break;
-    case DocInclude::SnipWithLines:
-      {
-         forceEndParagraph(inc);
-         m_ci.startCodeFragment("DoxyCode");
-         FileInfo cfi( inc.file().str() );
-         FileDef *fd = createFileDef( cfi.dirPath(), cfi.fileName() );
-         getCodeParser(inc.extension()).parseCode(m_ci,
-                                           inc.context(),
-                                           extractBlock(inc.text(),inc.blockId()),
-                                           langExt,
-                                           inc.isExample(),
-                                           inc.exampleFile(),
-                                           fd,
-                                           lineBlock(inc.text(),inc.blockId()),
-                                           -1,    // endLine
-                                           FALSE, // inlineFragment
-                                           0,     // memberDef
-                                           TRUE,  // show line number
-                                           m_ctx  // search context
-                                          );
-         delete fd;
-         m_ci.endCodeFragment("DoxyCode");
-         forceStartParagraph(inc);
-      }
-      break;
-    case DocInclude::SnippetDoc:
-    case DocInclude::IncludeDoc:
-      err("Internal inconsistency: found switch SnippetDoc / IncludeDoc in file: %s"
-          "Please create a bug report\n",__FILE__);
+    case DocInclude::SnippetTrimLeft:
+    case DocInclude::SnippetWithLines:
+      forceEndParagraph(inc);
+      m_ci.startCodeFragment("DoxyCode");
+      CodeFragmentManager::instance().parseCodeFragment(m_ci,
+                                       inc.file(),
+                                       inc.blockId(),
+                                       inc.context(),
+                                       inc.type()==DocInclude::SnippetWithLines,
+                                       inc.type()==DocInclude::SnippetTrimLeft
+                                      );
+      m_ci.endCodeFragment("DoxyCode");
+      forceStartParagraph(inc);
       break;
   }
 }
@@ -836,7 +832,7 @@ void HtmlDocVisitor::operator()(const DocIncOperator &op)
     m_hide = popHidden();
     if (!m_hide)
     {
-      FileDef *fd = 0;
+      std::unique_ptr<FileDef> fd;
       if (!op.includeFileName().isEmpty())
       {
         FileInfo cfi( op.includeFileName().str() );
@@ -849,15 +845,14 @@ void HtmlDocVisitor::operator()(const DocIncOperator &op)
                                 langExt,
                                 op.isExample(),
                                 op.exampleFile(),
-                                fd,     // fileDef
+                                fd.get(),     // fileDef
                                 op.line(),    // startLine
                                 -1,    // endLine
                                 FALSE, // inline fragment
-                                0,     // memberDef
+                                nullptr,     // memberDef
                                 op.showLineNo(),  // show line numbers
                                 m_ctx  // search context
                                );
-      if (fd) delete fd;
     }
     pushHidden(m_hide);
     m_hide=TRUE;
@@ -1056,7 +1051,14 @@ void HtmlDocVisitor::operator()(const DocAutoList &l)
   }
   else
   {
-    m_t << "<ul>";
+    if (l.isCheckedList())
+    {
+      m_t << "<ul class=\"check\">";
+    }
+    else
+    {
+      m_t << "<ul>";
+    }
   }
   if (!l.isPreformatted()) m_t << "\n";
   visitChildren(l);
@@ -1075,7 +1077,19 @@ void HtmlDocVisitor::operator()(const DocAutoList &l)
 void HtmlDocVisitor::operator()(const DocAutoListItem &li)
 {
   if (m_hide) return;
-  m_t << "<li>";
+  switch (li.itemNumber())
+  {
+    case DocAutoList::Unchecked: // unchecked
+      m_t << "<li class=\"unchecked\">";
+      break;
+    case DocAutoList::Checked_x: // checked with x
+    case DocAutoList::Checked_X: // checked with X
+      m_t << "<li class=\"checked\">";
+      break;
+    default:
+      m_t << "<li>";
+      break;
+  }
   visitChildren(li);
   m_t << "</li>";
   if (!li.isPreformatted()) m_t << "\n";
@@ -1110,7 +1124,7 @@ bool isSeparatedParagraph(const DocSimpleSect &parent,const DocPara &par)
   auto it = std::find_if(std::begin(nodes),std::end(nodes),[&par](const auto &n) { return holds_value(&par,n); });
   if (it==std::end(nodes)) return FALSE;
   size_t count = parent.children().size();
-  auto isSeparator = [](auto &&it_) { return std::get_if<DocSimpleSectSep>(&(*it_))!=0; };
+  auto isSeparator = [](auto &&it_) { return std::get_if<DocSimpleSectSep>(&(*it_))!=nullptr; };
   if (count>1 && it==std::begin(nodes)) // it points to first node
   {
     return isSeparator(std::next(it));
@@ -1126,9 +1140,9 @@ bool isSeparatedParagraph(const DocSimpleSect &parent,const DocPara &par)
   return false;
 }
 
-static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
+static contexts_t getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
 {
-  int t=0;
+  contexts_t t=contexts_t::NONE;
   isFirst=FALSE;
   isLast=FALSE;
   if (p.parent())
@@ -1138,28 +1152,28 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       // hierarchy: node N -> para -> parblock -> para
       // adapt return value to kind of N
-      const DocNodeVariant *p3 = 0;
+      const DocNodeVariant *p3 = nullptr;
       if (::parent(p.parent()) && ::parent(::parent(p.parent())) )
       {
         p3 = ::parent(::parent(p.parent()));
       }
       isFirst=isFirstChildNode(parBlock,p);
       isLast =isLastChildNode (parBlock,p);
-      bool isLI = p3!=0 && holds_one_of_alternatives<DocHtmlListItem,DocSecRefItem>(*p3);
-      bool isDD = p3!=0 && holds_one_of_alternatives<DocHtmlDescData,DocXRefItem,DocSimpleSect>(*p3);
-      bool isTD = p3!=0 && holds_one_of_alternatives<DocHtmlCell,DocParamList>(*p3);
-      t=NONE;
+      bool isLI = p3!=nullptr && holds_one_of_alternatives<DocHtmlListItem,DocSecRefItem>(*p3);
+      bool isDD = p3!=nullptr && holds_one_of_alternatives<DocHtmlDescData,DocXRefItem,DocSimpleSect>(*p3);
+      bool isTD = p3!=nullptr && holds_one_of_alternatives<DocHtmlCell,DocParamList>(*p3);
+      t=contexts_t::NONE;
       if (isFirst)
       {
-        if (isLI) t=STARTLI; else if (isDD) t=STARTDD; else if (isTD) t=STARTTD;
+        if (isLI) t=contexts_t::STARTLI; else if (isDD) t=contexts_t::STARTDD; else if (isTD) t=contexts_t::STARTTD;
       }
       if (isLast)
       {
-        if (isLI) t=ENDLI;   else if (isDD) t=ENDDD;   else if (isTD) t=ENDTD;
+        if (isLI) t=contexts_t::ENDLI;   else if (isDD) t=contexts_t::ENDDD;   else if (isTD) t=contexts_t::ENDTD;
       }
       if (!isFirst && !isLast)
       {
-        if (isLI) t=INTERLI; else if (isDD) t=INTERDD; else if (isTD) t=INTERTD;
+        if (isLI) t=contexts_t::INTERLI; else if (isDD) t=contexts_t::INTERDD; else if (isTD) t=contexts_t::INTERTD;
       }
       return t;
     }
@@ -1168,7 +1182,7 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=isFirstChildNode(docAutoListItem,p);
       isLast =isLastChildNode (docAutoListItem,p);
-      t=STARTLI; // not used
+      t=contexts_t::STARTLI; // not used
       return t;
     }
     const auto docSimpleListItem = std::get_if<DocSimpleListItem>(p.parent());
@@ -1176,7 +1190,7 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=TRUE;
       isLast =TRUE;
-      t=STARTLI; // not used
+      t=contexts_t::STARTLI; // not used
       return t;
     }
     const auto docParamList = std::get_if<DocParamList>(p.parent());
@@ -1184,7 +1198,7 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=TRUE;
       isLast =TRUE;
-      t=STARTLI; // not used
+      t=contexts_t::STARTLI; // not used
       return t;
     }
     const auto docHtmlListItem = std::get_if<DocHtmlListItem>(p.parent());
@@ -1192,9 +1206,9 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=isFirstChildNode(docHtmlListItem,p);
       isLast =isLastChildNode (docHtmlListItem,p);
-      if (isFirst) t=STARTLI;
-      if (isLast)  t=ENDLI;
-      if (!isFirst && !isLast) t = INTERLI;
+      if (isFirst) t=contexts_t::STARTLI;
+      if (isLast)  t=contexts_t::ENDLI;
+      if (!isFirst && !isLast) t = contexts_t::INTERLI;
       return t;
     }
     const auto docSecRefItem = std::get_if<DocSecRefItem>(p.parent());
@@ -1202,9 +1216,9 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=isFirstChildNode(docSecRefItem,p);
       isLast =isLastChildNode (docSecRefItem,p);
-      if (isFirst) t=STARTLI;
-      if (isLast)  t=ENDLI;
-      if (!isFirst && !isLast) t = INTERLI;
+      if (isFirst) t=contexts_t::STARTLI;
+      if (isLast)  t=contexts_t::ENDLI;
+      if (!isFirst && !isLast) t = contexts_t::INTERLI;
       return t;
     }
     const auto docHtmlDescData = std::get_if<DocHtmlDescData>(p.parent());
@@ -1212,9 +1226,9 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=isFirstChildNode(docHtmlDescData,p);
       isLast =isLastChildNode (docHtmlDescData,p);
-      if (isFirst) t=STARTDD;
-      if (isLast)  t=ENDDD;
-      if (!isFirst && !isLast) t = INTERDD;
+      if (isFirst) t=contexts_t::STARTDD;
+      if (isLast)  t=contexts_t::ENDDD;
+      if (!isFirst && !isLast) t = contexts_t::INTERDD;
       return t;
     }
     const auto docXRefItem = std::get_if<DocXRefItem>(p.parent());
@@ -1222,9 +1236,9 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=isFirstChildNode(docXRefItem,p);
       isLast =isLastChildNode (docXRefItem,p);
-      if (isFirst) t=STARTDD;
-      if (isLast)  t=ENDDD;
-      if (!isFirst && !isLast) t = INTERDD;
+      if (isFirst) t=contexts_t::STARTDD;
+      if (isLast)  t=contexts_t::ENDDD;
+      if (!isFirst && !isLast) t = contexts_t::INTERDD;
       return t;
     }
     const auto docSimpleSect = std::get_if<DocSimpleSect>(p.parent());
@@ -1232,8 +1246,8 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=isFirstChildNode(docSimpleSect,p);
       isLast =isLastChildNode (docSimpleSect,p);
-      if (isFirst) t=STARTDD;
-      if (isLast)  t=ENDDD;
+      if (isFirst) t=contexts_t::STARTDD;
+      if (isLast)  t=contexts_t::ENDDD;
       if (isSeparatedParagraph(*docSimpleSect,p))
         // if the paragraph is enclosed with separators it will
         // be included in <dd>..</dd> so avoid addition paragraph
@@ -1241,7 +1255,7 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
       {
         isFirst=isLast=TRUE;
       }
-      if (!isFirst && !isLast) t = INTERDD;
+      if (!isFirst && !isLast) t = contexts_t::INTERDD;
       return t;
     }
     const auto docHtmlCell = std::get_if<DocHtmlCell>(p.parent());
@@ -1249,9 +1263,9 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=isFirstChildNode(docHtmlCell,p);
       isLast =isLastChildNode (docHtmlCell,p);
-      if (isFirst) t=STARTTD;
-      if (isLast)  t=ENDTD;
-      if (!isFirst && !isLast) t = INTERTD;
+      if (isFirst) t=contexts_t::STARTTD;
+      if (isLast)  t=contexts_t::ENDTD;
+      if (!isFirst && !isLast) t = contexts_t::INTERTD;
       return t;
     }
   }
@@ -1320,10 +1334,9 @@ void HtmlDocVisitor::operator()(const DocPara &p)
   // check if this paragraph is the first or last or intermediate child of a <li> or <dd>.
   // this allows us to mark the tag with a special class so we can
   // fix the otherwise ugly spacing.
-  int t;
-  bool isFirst;
-  bool isLast;
-  t = getParagraphContext(p,isFirst,isLast);
+  bool isFirst = false;
+  bool isLast  = false;
+  contexts_t t = getParagraphContext(p,isFirst,isLast);
   //printf("startPara first=%d last=%d\n",isFirst,isLast);
   if (isFirst && isLast) needsTagBefore=FALSE;
 
@@ -1331,8 +1344,8 @@ void HtmlDocVisitor::operator()(const DocPara &p)
   // write the paragraph tag (if needed)
   if (needsTagBefore)
   {
-    if (strlen(contexts[t]))
-      m_t << "<p class=\"" << contexts[t] << "\"" << htmlAttribsToString(p.attribs()) << ">";
+    if (contexts(t))
+      m_t << "<p class=\"" << contexts(t) << "\"" << htmlAttribsToString(p.attribs()) << ">";
     else
       m_t << "<p" << htmlAttribsToString(p.attribs()) << ">";
   }
@@ -1422,6 +1435,8 @@ void HtmlDocVisitor::operator()(const DocSimpleSect &s)
       m_t << theTranslator->trRemarks(); break;
     case DocSimpleSect::Attention:
       m_t << theTranslator->trAttention(); break;
+    case DocSimpleSect::Important:
+      m_t << theTranslator->trImportant(); break;
     case DocSimpleSect::User: break;
     case DocSimpleSect::Rcs: break;
     case DocSimpleSect::Unknown:  break;
@@ -1474,7 +1489,10 @@ void HtmlDocVisitor::operator()(const DocSection &s)
   m_t << "<h" << s.level() << ">";
   m_t << "<a class=\"anchor\" id=\"" << s.anchor();
   m_t << "\"></a>\n";
-  filter(convertCharEntitiesToUTF8(s.title()));
+  if (s.title())
+  {
+    std::visit(*this,*s.title());
+  }
   m_t << "</h" << s.level() << ">\n";
   visitChildren(s);
   forceStartParagraph(s);
@@ -1674,12 +1692,7 @@ void HtmlDocVisitor::operator()(const DocImage &img)
       forceEndParagraph(img);
     }
     if (m_hide) return;
-    QCString baseName=img.name();
-    int i;
-    if ((i=baseName.findRev('/'))!=-1 || (i=baseName.findRev('\\'))!=-1)
-    {
-      baseName=baseName.right(baseName.length()-i-1);
-    }
+    QCString baseName=makeShortName(img.name());
     if (!inlineImage) m_t << "<div class=\"image\">\n";
     QCString sizeAttribs;
     if (!img.width().isEmpty())
@@ -1783,6 +1796,7 @@ void HtmlDocVisitor::operator()(const DocDotFile &df)
 {
   if (m_hide) return;
   if (!Config_getBool(DOT_CLEANUP)) copyFile(df.file(),Config_getString(HTML_OUTPUT)+"/"+stripPath(df.file()));
+  forceEndParagraph(df);
   m_t << "<div class=\"dotgraph\">\n";
   writeDotFile(df.file(),df.relPath(),df.context(),df.srcFile(),df.srcLine());
   if (df.hasCaption())
@@ -1795,12 +1809,14 @@ void HtmlDocVisitor::operator()(const DocDotFile &df)
     m_t << "</div>\n";
   }
   m_t << "</div>\n";
+  forceStartParagraph(df);
 }
 
 void HtmlDocVisitor::operator()(const DocMscFile &df)
 {
   if (m_hide) return;
   if (!Config_getBool(DOT_CLEANUP)) copyFile(df.file(),Config_getString(HTML_OUTPUT)+"/"+stripPath(df.file()));
+  forceEndParagraph(df);
   m_t << "<div class=\"mscgraph\">\n";
   writeMscFile(df.file(),df.relPath(),df.context(),df.srcFile(),df.srcLine());
   if (df.hasCaption())
@@ -1813,12 +1829,14 @@ void HtmlDocVisitor::operator()(const DocMscFile &df)
     m_t << "</div>\n";
   }
   m_t << "</div>\n";
+  forceStartParagraph(df);
 }
 
 void HtmlDocVisitor::operator()(const DocDiaFile &df)
 {
   if (m_hide) return;
   if (!Config_getBool(DOT_CLEANUP)) copyFile(df.file(),Config_getString(HTML_OUTPUT)+"/"+stripPath(df.file()));
+  forceEndParagraph(df);
   m_t << "<div class=\"diagraph\">\n";
   writeDiaFile(df.file(),df.relPath(),df.context(),df.srcFile(),df.srcLine());
   if (df.hasCaption())
@@ -1831,6 +1849,7 @@ void HtmlDocVisitor::operator()(const DocDiaFile &df)
     m_t << "</div>\n";
   }
   m_t << "</div>\n";
+  forceStartParagraph(df);
 }
 
 void HtmlDocVisitor::operator()(const DocLink &lnk)
@@ -1989,8 +2008,10 @@ void HtmlDocVisitor::operator()(const DocXRefItem &x)
   bool anonymousEnum = x.file()=="@";
   if (!anonymousEnum)
   {
+    QCString fn = x.file();
+    addHtmlExtensionIfMissing(fn);
     m_t << "<dl class=\"" << x.key() << "\"><dt><b><a class=\"el\" href=\""
-        << x.relPath() << addHtmlExtensionIfMissing(x.file())
+        << x.relPath() << fn
         << "#" << x.anchor() << "\">";
   }
   else
@@ -1998,7 +2019,6 @@ void HtmlDocVisitor::operator()(const DocXRefItem &x)
     m_t << "<dl class=\"" << x.key() << "\"><dt><b>";
   }
   filter(x.title());
-  m_t << ":";
   if (!anonymousEnum) m_t << "</a>";
   m_t << "</b></dt><dd>";
   visitChildren(x);
@@ -2040,6 +2060,7 @@ void HtmlDocVisitor::operator()(const DocVhdlFlow &vf)
     QCString fname=FlowChart::convertNameToFileName();
     m_t << "<p>";
     m_t << theTranslator->trFlowchart();
+    m_t << " ";
     m_t << "<a href=\"";
     m_t << fname;
     m_t << ".svg\">";
@@ -2068,10 +2089,9 @@ void HtmlDocVisitor::filter(const QCString &str, const bool retainNewline)
 {
   if (str.isEmpty()) return;
   const char *p=str.data();
-  char c;
   while (*p)
   {
-    c=*p++;
+    char c=*p++;
     switch(c)
     {
       case '\n': if(retainNewline) m_t << "<br/>"; m_t << c; break;
@@ -2085,7 +2105,7 @@ void HtmlDocVisitor::filter(const QCString &str, const bool retainNewline)
         break;
       default:
         {
-          uchar uc = static_cast<uchar>(c);
+          uint8_t uc = static_cast<uint8_t>(c);
           if (uc<32 && !isspace(c)) // non-printable control characters
           {
             m_t << "&#x24" << hex[uc>>4] << hex[uc&0xF] << ";";
@@ -2107,10 +2127,9 @@ QCString HtmlDocVisitor::filterQuotedCdataAttr(const QCString &str)
   GrowBuf growBuf;
   if (str.isEmpty()) return str;
   const char *p=str.data();
-  char c;
   while (*p)
   {
-    c=*p++;
+    char c=*p++;
     switch(c)
     {
       case '&':  growBuf.addStr("&amp;"); break;
@@ -2130,7 +2149,7 @@ QCString HtmlDocVisitor::filterQuotedCdataAttr(const QCString &str)
         break;
       default:
         {
-          uchar uc = static_cast<uchar>(c);
+          uint8_t uc = static_cast<uint8_t>(c);
           if (uc<32 && !isspace(c)) // non-printable control characters
           {
             growBuf.addStr("&#x24");
@@ -2165,12 +2184,12 @@ void HtmlDocVisitor::startLink(const QCString &ref,const QCString &file,
     m_t << "<a class=\"el\" ";
   }
   m_t << "href=\"";
-  m_t << externalRef(relPath,ref,TRUE);
-  if (!file.isEmpty())
-  {
-    m_t << addHtmlExtensionIfMissing(file);
-  }
-  if (!anchor.isEmpty()) m_t << "#" << anchor;
+  QCString fn = file;
+  addHtmlExtensionIfMissing(fn);
+  m_t << createHtmlUrl(relPath,ref,true,
+                       m_fileName == Config_getString(HTML_OUTPUT)+"/"+fn,
+                       fn,
+                       anchor);
   m_t << "\"";
   if (!tooltip.isEmpty()) m_t << " title=\"" << convertToHtml(tooltip) << "\"";
   m_t << ">";
@@ -2184,16 +2203,7 @@ void HtmlDocVisitor::endLink()
 void HtmlDocVisitor::writeDotFile(const QCString &fn,const QCString &relPath,
                                   const QCString &context,const QCString &srcFile,int srcLine)
 {
-  QCString baseName=fn;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
-  if ((i=baseName.find('.'))!=-1) // strip extension
-  {
-    baseName=baseName.left(i);
-  }
+  QCString baseName=makeBaseName(fn);
   baseName.prepend("dot_");
   QCString outDir = Config_getString(HTML_OUTPUT);
   writeDotGraphFromFile(fn,outDir,baseName,GOF_BITMAP,srcFile,srcLine);
@@ -2203,16 +2213,7 @@ void HtmlDocVisitor::writeDotFile(const QCString &fn,const QCString &relPath,
 void HtmlDocVisitor::writeMscFile(const QCString &fileName,const QCString &relPath,
                                   const QCString &context,const QCString &srcFile,int srcLine)
 {
-  QCString baseName=fileName;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1) // strip path
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
-  if ((i=baseName.find('.'))!=-1) // strip extension
-  {
-    baseName=baseName.left(i);
-  }
+  QCString baseName=makeBaseName(fileName);
   baseName.prepend("msc_");
   QCString outDir = Config_getString(HTML_OUTPUT);
   QCString imgExt = getDotImageExtension();
@@ -2226,16 +2227,7 @@ void HtmlDocVisitor::writeMscFile(const QCString &fileName,const QCString &relPa
 void HtmlDocVisitor::writeDiaFile(const QCString &fileName, const QCString &relPath,
                                   const QCString &,const QCString &srcFile,int srcLine)
 {
-  QCString baseName=fileName;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1) // strip path
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
-  if ((i=baseName.find('.'))!=-1) // strip extension
-  {
-    baseName=baseName.left(i);
-  }
+  QCString baseName=makeBaseName(fileName);
   baseName.prepend("dia_");
   QCString outDir = Config_getString(HTML_OUTPUT);
   writeDiaGraphFromFile(fileName,outDir,baseName,DIA_BITMAP,srcFile,srcLine);
@@ -2244,18 +2236,9 @@ void HtmlDocVisitor::writeDiaFile(const QCString &fileName, const QCString &relP
 }
 
 void HtmlDocVisitor::writePlantUMLFile(const QCString &fileName, const QCString &relPath,
-                                       const QCString &,const QCString &srcFile,int srcLine)
+                                       const QCString &,const QCString &/* srcFile */,int /* srcLine */)
 {
-  QCString baseName=fileName;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1) // strip path
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
-  if ((i=baseName.findRev('.'))!=-1) // strip extension
-  {
-    baseName=baseName.left(i);
-  }
+  QCString baseName=makeBaseName(fileName);
   QCString outDir = Config_getString(HTML_OUTPUT);
   QCString imgExt = getDotImageExtension();
   if (imgExt=="svg")
@@ -2360,8 +2343,8 @@ void HtmlDocVisitor::forceEndParagraph(const Node &n)
       it = std::prev(it);
       styleOutsideParagraph=insideStyleChangeThatIsOutsideParagraph(para,it);
     }
-    bool isFirst;
-    bool isLast;
+    bool isFirst = false;
+    bool isLast = false;
     getParagraphContext(*para,isFirst,isLast);
     //printf("forceEnd first=%d last=%d styleOutsideParagraph=%d\n",isFirst,isLast,styleOutsideParagraph);
     if (isFirst && isLast) return;
@@ -2380,7 +2363,7 @@ template<class Node>
 void HtmlDocVisitor::forceStartParagraph(const Node &n)
 {
   //printf("> forceStartParagraph(%s)\n",docNodeName(n));
-  const DocPara *para=0;
+  const DocPara *para=nullptr;
   if (n.parent() && (para = std::get_if<DocPara>(n.parent()))) // if we are inside a paragraph
   {
     const DocNodeList &children = para->children();
@@ -2409,11 +2392,11 @@ void HtmlDocVisitor::forceStartParagraph(const Node &n)
       return; // only whitespace at the end!
     }
 
-    bool needsTag = TRUE;
-    bool isFirst;
-    bool isLast;
+    bool needsTag = true;
+    bool isFirst = false;
+    bool isLast = false;
     getParagraphContext(*para,isFirst,isLast);
-    if (isFirst && isLast) needsTag = FALSE;
+    if (isFirst && isLast) needsTag = false;
     //printf("forceStart first=%d last=%d needsTag=%d\n",isFirst,isLast,needsTag);
 
     if (needsTag) m_t << "<p>";
