@@ -182,14 +182,14 @@ static void do_warn(bool enabled, const QCString &file, int line, const char *pr
   // format + arguments
   // prefix
   // 1 position for `\0`
-  size_t bufSize = vsnprintf(NULL, 0, fmt, args) + l + 1;
-  QCString text(bufSize);
+  size_t bufSize = vsnprintf(nullptr, 0, fmt, args) + l;
+  QCString text(bufSize, QCString::ExplicitSize);
   if (prefix)
   {
     qstrncpy(text.rawData(),prefix,bufSize);
   }
-  vsnprintf(text.rawData()+l, bufSize-l, fmt, argsCopy);
-  text[static_cast<int>(bufSize)-1]='\0';
+  vsnprintf(text.rawData()+l, bufSize-l+1, fmt, argsCopy);
+  text[bufSize]='\0';
   format_warn(file,line,text);
 
   va_end(argsCopy);
@@ -207,7 +207,7 @@ QCString warn_line(const QCString &file,int line)
             "$line",lineSubst
           );
 }
-void warn(const QCString &file,int line,const char *fmt, ...)
+void warn_(const QCString &file,int line,const char *fmt, ...)
 {
   va_list args;
   va_start(args, fmt);
@@ -220,13 +220,7 @@ void va_warn(const QCString &file,int line,const char *fmt,va_list args)
   do_warn(Config_getBool(WARNINGS), file, line, g_warningStr, fmt, args);
 }
 
-void warn_simple(const QCString &file,int line,const char *text)
-{
-  if (!Config_getBool(WARNINGS)) return; // warning type disabled
-  format_warn(file,line,QCString(g_warningStr) + text);
-}
-
-void warn_undoc(const QCString &file,int line,const char *fmt, ...)
+void warn_undoc_(const QCString &file,int line,const char *fmt, ...)
 {
   va_list args;
   va_start(args, fmt);
@@ -234,7 +228,7 @@ void warn_undoc(const QCString &file,int line,const char *fmt, ...)
   va_end(args);
 }
 
-void warn_incomplete_doc(const QCString &file,int line,const char *fmt, ...)
+void warn_incomplete_doc_(const QCString &file,int line,const char *fmt, ...)
 {
   va_list args;
   va_start(args, fmt);
@@ -242,7 +236,7 @@ void warn_incomplete_doc(const QCString &file,int line,const char *fmt, ...)
   va_end(args);
 }
 
-void warn_doc_error(const QCString &file,int line,const char *fmt, ...)
+void warn_doc_error_(const QCString &file,int line,const char *fmt, ...)
 {
   va_list args;
   va_start(args, fmt);
@@ -250,7 +244,7 @@ void warn_doc_error(const QCString &file,int line,const char *fmt, ...)
   va_end(args);
 }
 
-void warn_uncond(const char *fmt, ...)
+void warn_uncond_(const char *fmt, ...)
 {
   va_list args;
   va_start(args, fmt);
@@ -259,7 +253,7 @@ void warn_uncond(const char *fmt, ...)
   handle_warn_as_error();
 }
 
-void err(const char *fmt, ...)
+void err_(const char *fmt, ...)
 {
   va_list args;
   va_start(args, fmt);
@@ -268,7 +262,7 @@ void err(const char *fmt, ...)
   handle_warn_as_error();
 }
 
-extern void err_full(const QCString &file,int line,const char *fmt, ...)
+extern void err_full_(const QCString &file,int line,const char *fmt, ...)
 {
   va_list args;
   va_start(args, fmt);
@@ -276,7 +270,7 @@ extern void err_full(const QCString &file,int line,const char *fmt, ...)
   va_end(args);
 }
 
-void term(const char *fmt, ...)
+void term_(const char *fmt, ...)
 {
   {
     std::unique_lock<std::mutex> lock(g_mutex);
@@ -304,6 +298,7 @@ void warn_flush()
 
 extern void finishWarnExit()
 {
+  fflush(stdout);
   if (g_warnBehavior == WARN_AS_ERROR_t::FAIL_ON_WARNINGS_PRINT && g_warnlogFile != "-")
   {
     Portable::fclose(g_warnFile);
